/* BdfServer - Copyright (c) 2009-2019 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.balayage;

import fr.exemole.bdfserver.api.balayage.Balayage;
import fr.exemole.bdfserver.api.balayage.BalayageConstants;
import fr.exemole.bdfserver.api.balayage.BalayageMode;
import fr.exemole.bdfserver.api.balayage.BalayageUnit;
import fr.exemole.bdfserver.api.balayage.SiteMapOption;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import net.fichotheque.utils.SelectionUtils;
import net.mapeadores.util.localisation.LangContext;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.DocumentFragmentHolder;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import net.fichotheque.selection.FicheQuery;


/**
 *
 * @author Vincent Calame
 */
public class BalayageBuilder implements BalayageConstants {

    private final static short RESOLVED_ALL = 1;
    private final static short RESOLVED_SOME = 2;
    private final static short RESOLVED_NONE = 3;
    private final InternalBalayage balayage;
    private final List<FicheQuery> globalFicheQuery = new ArrayList<FicheQuery>();
    private final Map<String, InternalBalayageMode> modeMap = new LinkedHashMap<String, InternalBalayageMode>();
    private final List<BalayageUnitBuilder> unitList = new ArrayList<BalayageUnitBuilder>();
    private final IncludeCatalogBuilder includeCatalogBuilder = new IncludeCatalogBuilder();

    public BalayageBuilder(String name) {
        balayage = new InternalBalayage(name);
    }

    public Balayage toBalayage() {
        balayage.globalFicheQueryList = SelectionUtils.wrap(globalFicheQuery.toArray(new FicheQuery[globalFicheQuery.size()]));
        int size2 = modeMap.size();
        if (size2 > 0) {
            BalayageMode[] array = new BalayageMode[size2];
            int p = 0;
            for (BalayageMode balayageMode : modeMap.values()) {
                array[p] = balayageMode;
                p++;
            }
            balayage.balayageModeArray = array;
        }
        int size3 = unitList.size();
        if (size3 > 0) {
            BalayageUnit[] balayageUnitArray = new BalayageUnit[size3];
            for (int i = 0; i < size3; i++) {
                balayageUnitArray[i] = unitList.get(i).toBalayageUnit();
            }
            balayage.balayageUnitArray = balayageUnitArray;
        }
        balayage.includeCatalog = includeCatalogBuilder.toIncludeCatalog();
        return balayage;
    }

    public void addGlobalFicheQuery(FicheQuery ficheQuery) {
        globalFicheQuery.add(ficheQuery);
    }

    public void setRootName(String rootName) {
        balayage.rootName = rootName;
    }

    public void setPath(String path) {
        balayage.path = path;
    }

    public void setWithTransformation(boolean withTransformation) {
        balayage.withTransformation = withTransformation;
    }

    public void setPostscriptumCommand(String command) {
        balayage.postscriptum.setCommand(command);
    }

    public void addPostscriptumScrutariExport(String name) {
        balayage.postscriptum.addScrutariExport(name);
    }

    public void addPostscriptumSqlExport(String name) {
        balayage.postscriptum.addSqlExport(name);
    }

    public void setDefaultLangOption(short defaultLangOption) {
        balayage.defaultLangOption = defaultLangOption;
    }

    public void setLangContext(LangContext langContext) {
        balayage.langContext = langContext;
    }

    public void setSiteMapOption(SiteMapOption siteMapOption) {
        balayage.siteMapOption = siteMapOption;
    }

    public BalayageMode addBalayageMode(String name) {
        InternalBalayageMode ibe = modeMap.get(name);
        if (ibe == null) {
            ibe = new InternalBalayageMode(name);
            modeMap.put(name, ibe);
        }
        return ibe;
    }

    public void setWithCurrentDateLimit(BalayageMode balayageMode, boolean b) {
        ((InternalBalayageMode) balayageMode).setWithCurrentDateLimit(b);
    }

    public BalayageUnitBuilder addBalayageUnitBuilder(short type) {
        BalayageUnitBuilder unitBuilder = new BalayageUnitBuilder(type);
        unitList.add(unitBuilder);
        return unitBuilder;
    }

    public void addIncludeFragment(String key, NodeList nodeList) {
        includeCatalogBuilder.addIncludeFragment(key, nodeList);
    }


    private static class InternalBalayage implements Balayage {

        private short defaultLangOption = LANG_OPTION_NONE;
        private final String name;
        private LangContext langContext;
        private boolean withTransformation;
        private String rootName;
        private String path;
        private List<FicheQuery> globalFicheQueryList;
        private BalayageMode[] balayageModeArray;
        private BalayageUnit[] balayageUnitArray;
        private SiteMapOption siteMapOption;
        private InternalPostscriptum postscriptum;
        private DocumentFragmentHolder includeCatalog;

        private InternalBalayage(String name) {
            this.name = name;
            this.postscriptum = new InternalPostscriptum();
        }

        @Override
        public String getName() {
            return name;
        }

        @Override
        public LangContext getLangContext() {
            return langContext;
        }

        @Override
        public String getRootName() {
            return rootName;
        }

        @Override
        public String getPath() {
            return path;
        }

        @Override
        public boolean isWithTransformation() {
            return withTransformation;
        }

        @Override
        public int getBalayageUnitCount() {
            if (balayageUnitArray == null) {
                return 0;
            }
            return balayageUnitArray.length;
        }

        @Override
        public BalayageUnit getBalayageUnit(int i) {
            return balayageUnitArray[i];
        }

        @Override
        public List<FicheQuery> getGlobalFicheQueryList() {
            return globalFicheQueryList;
        }

        @Override
        public int getBalayageModeCount() {
            if (balayageModeArray == null) {
                return 0;
            }
            return balayageModeArray.length;
        }

        @Override
        public BalayageMode getBalayageMode(int i) {
            return balayageModeArray[i];
        }

        @Override
        public short getDefaultLangOption() {
            return defaultLangOption;
        }

        @Override
        public SiteMapOption getSiteMapOption() {
            return siteMapOption;
        }

        @Override
        public Postscriptum getPostscriptum() {
            return postscriptum;
        }

        @Override
        public DocumentFragmentHolder getIncludeCatalog() {
            return includeCatalog;
        }

    }


    private static class InternalPostscriptum implements Balayage.Postscriptum {

        private String command;
        private final List<String> scrutariExportList = new ArrayList<String>();
        private final List<String> sqlExportList = new ArrayList<String>();

        private InternalPostscriptum() {
        }

        @Override
        public String getCommand() {
            return command;
        }

        @Override
        public int getScrutariExportCount() {
            return scrutariExportList.size();
        }

        @Override
        public String getScrutariExportName(int i) {
            return scrutariExportList.get(i);
        }

        @Override
        public int getSqlExportCount() {
            return sqlExportList.size();
        }

        @Override
        public String getSqlExportName(int i) {
            return sqlExportList.get(i);
        }

        private void setCommand(String command) {
            this.command = command;
        }

        private void addScrutariExport(String name) {
            scrutariExportList.add(name);
        }

        private void addSqlExport(String name) {
            sqlExportList.add(name);
        }

    }


    private static class InternalBalayageMode implements BalayageMode {

        private final String name;
        private boolean withCurrentDateLimit = false;

        InternalBalayageMode(String name) {
            this.name = name;
        }

        @Override
        public String getName() {
            return name;
        }

        @Override
        public boolean isWithCurrentDateLimit() {
            return withCurrentDateLimit;
        }

        private void setWithCurrentDateLimit(boolean withCurrentDateLimit) {
            this.withCurrentDateLimit = withCurrentDateLimit;
        }

    }


    private static class IncludeCatalogBuilder {

        private final Document document = DOMUtils.newDocument();
        private final Map<String, DocumentFragment> finalFragmentMap = new HashMap<String, DocumentFragment>();
        private final Map<String, DocumentFragmentBuilder> unresolvedFragmentMap = new HashMap<String, DocumentFragmentBuilder>();

        private IncludeCatalogBuilder() {
        }

        public DocumentFragmentHolder toIncludeCatalog() {
            resolve();
            return new InternalIncludeCatalog(finalFragmentMap);
        }

        private void resolve() {
            if (unresolvedFragmentMap.isEmpty()) {
                return;
            }
            boolean oneDone = false;
            for (Iterator<DocumentFragmentBuilder> it = unresolvedFragmentMap.values().iterator(); it.hasNext();) {
                DocumentFragmentBuilder documentFragmentBuilder = it.next();
                short result = documentFragmentBuilder.resolve(finalFragmentMap);
                switch (result) {
                    case RESOLVED_ALL:
                        it.remove();
                        oneDone = true;
                        break;
                    case RESOLVED_SOME:
                        oneDone = true;
                        break;
                    case RESOLVED_NONE:

                }
            }
            if (oneDone) {
                if (unresolvedFragmentMap.isEmpty()) {
                    return;
                } else {
                    resolve();
                }
            } else {
                //@todo avertissement toutes les inclusions n'ont pas été résolues
                return;
            }
        }

        private void addIncludeFragment(String key, NodeList nodeList) {
            if (finalFragmentMap.containsKey(key)) {
                finalFragmentMap.remove(key);
            }
            if (unresolvedFragmentMap.containsKey(key)) {
                unresolvedFragmentMap.remove(key);
            }
            DocumentFragment documentFragment = document.createDocumentFragment();
            int count = nodeList.getLength();
            for (int i = 0; i < count; i++) {
                Node node = nodeList.item(i);
                Node importNode = document.importNode(node, true);
                documentFragment.appendChild(importNode);
            }
            DocumentFragmentBuilder documentFragmentBuilder = new DocumentFragmentBuilder(key, documentFragment);
            if (documentFragmentBuilder.hasIncludeElement()) {
                unresolvedFragmentMap.put(key, documentFragmentBuilder);
            } else {
                finalFragmentMap.put(key, documentFragment);
            }
        }

    }


    private static class DocumentFragmentBuilder {

        private final String key;
        private final DocumentFragment documentFragment;
        private final List<Element> elementList = new LinkedList<Element>();

        private DocumentFragmentBuilder(String key, DocumentFragment documentFragment) {
            this.key = key;
            this.documentFragment = documentFragment;
            NodeList list = documentFragment.getChildNodes();
            int count = list.getLength();
            for (int i = 0; i < count; i++) {
                Node node = list.item(i);
                if (node instanceof Element) {
                    Element element = (Element) node;
                    if (element.getTagName().equals("include")) {
                        addIncludeElement(element);
                    } else {
                        NodeList includeChildren = element.getElementsByTagName("include");
                        int includeCount = includeChildren.getLength();
                        for (int j = 0; j < includeCount; j++) {
                            addIncludeElement((Element) includeChildren.item(j));
                        }
                    }
                }
            }
        }

        private short resolve(Map<String, DocumentFragment> finalFragmentMap) {
            boolean oneDone = false;
            for (Iterator<Element> it = elementList.iterator(); it.hasNext();) {
                Element element = it.next();
                String fragmentKey = element.getAttribute("fragment-key");
                DocumentFragment finalFragment = finalFragmentMap.get(fragmentKey);
                if (finalFragment != null) {
                    finalFragment = (DocumentFragment) finalFragment.cloneNode(true);
                    element.getParentNode().replaceChild(finalFragment, element);
                    oneDone = true;
                    it.remove();
                }
            }
            if (oneDone) {
                if (elementList.isEmpty()) {
                    finalFragmentMap.put(key, documentFragment);
                    return RESOLVED_ALL;
                } else {
                    return RESOLVED_SOME;
                }
            } else {
                return RESOLVED_NONE;
            }
        }

        private boolean hasIncludeElement() {
            return (elementList.size() > 0);
        }

        private void addIncludeElement(Element element) {
            NodeList includeChildren = element.getElementsByTagName("include");
            if (includeChildren.getLength() == 0) {
                String fragmentKey = element.getAttribute("fragment-key");
                if (fragmentKey.length() > 0) {
                    elementList.add(element);
                }
            } else {
                //@todo : avertissement, un élément include ne doit pas contenir un autre élément include
            }
        }

    }


    private static class InternalIncludeCatalog implements DocumentFragmentHolder {

        private final Map<String, DocumentFragment> fragmentMap;

        private InternalIncludeCatalog(Map<String, DocumentFragment> fragmentMap) {
            this.fragmentMap = fragmentMap;
        }

        @Override
        public DocumentFragment getFragment(String fragmentKey) {
            return fragmentMap.get(fragmentKey);
        }

    }

}
