/* BdfServer - Copyright (c) 2015-2019 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.exportation.transformation;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.configuration.PathConfiguration;
import fr.exemole.bdfserver.tools.BdfURI;
import fr.exemole.bdfserver.tools.docstream.DocStreamFactory;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URI;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.opendocument.io.OdSource;
import net.mapeadores.opendocument.io.OdUtils;
import net.mapeadores.opendocument.io.PictureProvider;
import net.mapeadores.opendocument.transform.PictureProviderBuilder;
import net.mapeadores.util.io.DocStream;
import net.mapeadores.util.io.IOUtils;


/**
 *
 * @author Vincent Calame
 */
public class BdfPictureProviderBuilder extends PictureProviderBuilder {

    private final BdfServer bdfServer;
    private final PathConfiguration pathConfiguration;
    private final Map<URI, String> pictureHrefMap = new LinkedHashMap<URI, String>();
    private int current = 1;

    public BdfPictureProviderBuilder(BdfServer bdfServer, PathConfiguration pathConfiguration) {
        this.bdfServer = bdfServer;
        this.pathConfiguration = pathConfiguration;
    }

    @Override
    public String checkHref(String href) {
        URI uri = BdfURI.toURI(href, null);
        if (uri == null) {
            return null;
        }
        String scheme = uri.getScheme();
        if (!scheme.equals("bdf")) {
            return null;
        }
        String checkedHref = pictureHrefMap.get(uri);
        if (checkedHref == null) {
            int idx = href.lastIndexOf(".");
            int idx2 = href.lastIndexOf("/");
            if (idx2 > idx) {
                idx = -1;
            }
            if (idx != -1) {
                checkedHref = "Pictures/bdf-" + current + "." + href.substring(idx + 1);
            } else {
                checkedHref = "Pictures/bdf-" + current;
            }
            current++;
            pictureHrefMap.put(uri, checkedHref);
        }
        return checkedHref;
    }

    @Override
    public PictureProvider toPictureProvider() {
        int size = pictureHrefMap.size();
        PictureProvider.Picture[] pictureArray = new PictureProvider.Picture[size];
        int p = 0;
        for (Map.Entry<URI, String> entry : pictureHrefMap.entrySet()) {
            pictureArray[p] = new InternalPicture(bdfServer, pathConfiguration, entry.getValue(), entry.getKey());
            p++;
        }
        List<PictureProvider.Picture> pictureList = OdUtils.wrap(pictureArray);
        return new BdfPictureProvider(pictureList);
    }


    private static class BdfPictureProvider implements PictureProvider {

        private final List<PictureProvider.Picture> pictureList;

        private BdfPictureProvider(List<PictureProvider.Picture> pictureList) {
            this.pictureList = pictureList;
        }

        @Override
        public List<Picture> getPictureList() {
            return pictureList;
        }

    }


    private static class InternalPicture implements PictureProvider.Picture {

        private final BdfServer bdfServer;
        private PathConfiguration pathConfiguration;
        private final String href;
        private final URI uri;

        private InternalPicture(BdfServer bdfServer, PathConfiguration pathConfiguration, String href, URI uri) {
            this.bdfServer = bdfServer;
            this.pathConfiguration = pathConfiguration;
            this.href = href;
            this.uri = uri;
        }

        @Override
        public String getHref() {
            return href;
        }

        @Override
        public OdSource getOdSource() {
            DocStream docStream = DocStreamFactory.buildDocStream(bdfServer, pathConfiguration, uri);
            if (docStream != null) {
                return new DocStreamOdSource(docStream);
            }
            return OdUtils.EMPTY_ODSOURCE;
        }

    }


    private static class DocStreamOdSource implements OdSource {

        private final DocStream docStream;

        private DocStreamOdSource(DocStream docStream) {
            this.docStream = docStream;
        }

        @Override
        public void writeStream(OutputStream outputStream) throws IOException {
            try (InputStream is = docStream.getInputStream()) {
                IOUtils.copy(is, outputStream);
            }
        }

    }

}
