/* BdfServer - Copyright (c) 2009-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.ficheform;

import fr.exemole.bdfserver.api.ficheform.FicheForm;
import fr.exemole.bdfserver.api.ficheform.FicheFormParameters;
import fr.exemole.bdfserver.api.ficheform.FormElement;
import fr.exemole.bdfserver.api.ui.FieldUi;
import fr.exemole.bdfserver.api.ui.IncludeUi;
import fr.exemole.bdfserver.tools.ficheform.builders.AddendaIncludeElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.AlbumIncludeElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.CorpusIncludeElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.GeopointProprieteSubfieldsElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.ImageProprieteSubfieldsElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.LangFieldElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.MontantInformationSubfieldsElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.MontantProprieteSubfieldsElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.PersonneProprieteSubfieldElementBuilder;
import fr.exemole.bdfserver.tools.ficheform.builders.RattachementElementBuilder;
import java.util.HashSet;
import java.util.Set;
import net.fichotheque.FichothequeConstants;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.metadata.CorpusField;
import net.fichotheque.corpus.metadata.FieldGenerationDef;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.include.ExtendedIncludeKey;
import net.fichotheque.tools.corpus.FieldGenerationDefParser;
import net.fichotheque.utils.CorpusMetadataUtils;
import net.mapeadores.util.logging.SimpleLineMessageHandler;


/**
 *
 * @author Vincent Calame
 */
public class FicheFormBuilder {

    private FicheFormBuilder() {
    }

    public static FicheForm toFicheForm(FicheFormParameters ficheFormParameters) {
        return toFicheForm(ficheFormParameters, null);
    }

    public static FicheForm toFicheForm(FicheFormParameters ficheFormParameters, FicheForm overrideFicheForm) {
        return new InternalFicheForm(ficheFormParameters, overrideFicheForm);
    }


    private static class InternalFicheForm implements FicheForm {

        private final FicheFormParameters ficheFormParameters;
        private final FicheForm overrideFicheForm;
        private final Set<FieldKey> generatedSet = new HashSet<FieldKey>();

        public InternalFicheForm(FicheFormParameters ficheFormParameters, FicheForm overrideFicheForm) {
            this.ficheFormParameters = ficheFormParameters;
            this.overrideFicheForm = overrideFicheForm;
            String fieldGeneration = ficheFormParameters.getCorpus().getCorpusMetadata().getFieldGeneration();
            if (fieldGeneration != null) {
                for (FieldGenerationDef fieldGenerationDef : FieldGenerationDefParser.parse(fieldGeneration, new SimpleLineMessageHandler())) {
                    generatedSet.add(fieldGenerationDef.getFieldKey());
                }
            }
        }

        @Override
        public FormElement.Include getFormElement(IncludeUi includeUi) {
            if (overrideFicheForm != null) {
                FormElement.Include includeElement = overrideFicheForm.getFormElement(includeUi);
                if (includeElement != null) {
                    return includeElement;
                }
            }
            ExtendedIncludeKey extentedIncludeKey = includeUi.getExtendedIncludeKey();
            if (extentedIncludeKey == null) {
                switch (includeUi.getName()) {
                    case FichothequeConstants.LIAGE_NAME:
                        CorpusIncludeElementBuilder cieb = CorpusIncludeElementBuilder.check(ficheFormParameters, includeUi);
                        if (cieb != null) {
                            return cieb.toCorpusIncludeElement();
                        } else {
                            return null;
                        }
                    case FichothequeConstants.RATTACHEMENT_NAME:
                        RattachementElementBuilder reb = RattachementElementBuilder.check(ficheFormParameters, includeUi);
                        if (reb != null) {
                            return reb.toRattachementElement();
                        } else {
                            return null;
                        }
                    default:
                        return null;
                }
            } else {
                switch (extentedIncludeKey.getCategory()) {
                    case SubsetKey.CATEGORY_ADDENDA:
                        AddendaIncludeElementBuilder adieb = AddendaIncludeElementBuilder.check(ficheFormParameters, includeUi);
                        if (adieb != null) {
                            return adieb.toAddendaIncludeElement();
                        } else {
                            return null;
                        }
                    case SubsetKey.CATEGORY_ALBUM:
                        AlbumIncludeElementBuilder alieb = AlbumIncludeElementBuilder.check(ficheFormParameters, includeUi);
                        if (alieb != null) {
                            return alieb.toAlbumIncludeElement();
                        } else {
                            return null;
                        }
                    case SubsetKey.CATEGORY_CORPUS:
                        CorpusIncludeElementBuilder cieb = CorpusIncludeElementBuilder.check(ficheFormParameters, includeUi);
                        if (cieb != null) {
                            return cieb.toCorpusIncludeElement();
                        } else {
                            return null;
                        }
                    case SubsetKey.CATEGORY_THESAURUS:
                        return FicheFormUtils.toThesaurusIncludeElement(ficheFormParameters, includeUi);
                    default:
                        return null;
                }
            }
        }

        @Override
        public FormElement.Field getFormElement(FieldUi fieldUi) {
            if (overrideFicheForm != null) {
                FormElement.Field fieldElement = overrideFicheForm.getFormElement(fieldUi);
                if (fieldElement != null) {
                    return fieldElement;
                }
            }
            CorpusField corpusField = ficheFormParameters.checkCorpusField(fieldUi);
            if (corpusField == null) {
                return null;
            }
            FieldKey fieldKey = corpusField.getFieldKey();
            if (generatedSet.contains(fieldKey)) {
                return null;
            }
            if (fieldKey == FieldKey.ID) {
                return null;
            }
            if (fieldKey == FieldKey.LANG) {
                return LangFieldElementBuilder.build(ficheFormParameters, corpusField, fieldUi)
                        .toLangFieldElement();
            }
            if (CorpusMetadataUtils.hasPersonneSubfields(corpusField)) {
                return PersonneProprieteSubfieldElementBuilder.build(ficheFormParameters, corpusField, fieldUi)
                        .toPersonneProprieteSubfieldsElement();
            }
            if (CorpusMetadataUtils.hasGeopointSubfields(corpusField)) {
                return GeopointProprieteSubfieldsElementBuilder.build(ficheFormParameters, corpusField, fieldUi)
                        .toGeopointProprieteSubfieldsElement();
            }
            if (CorpusMetadataUtils.isCurrenciesPropriete(corpusField)) {
                return MontantProprieteSubfieldsElementBuilder.build(ficheFormParameters, corpusField, fieldUi)
                        .toMontantProprieteSubfieldsElement();
            }
            if (CorpusMetadataUtils.isCurrenciesInformation(corpusField)) {
                return MontantInformationSubfieldsElementBuilder.build(ficheFormParameters, corpusField, fieldUi)
                        .toMontantInformationSubfieldsElement();
            }
            if (CorpusMetadataUtils.hasImageSubfields(corpusField)) {
                return ImageProprieteSubfieldsElementBuilder.build(ficheFormParameters, corpusField, fieldUi)
                        .toImageProprieteSubfieldsElement();
            }
            return FicheFormUtils.toFormElement(ficheFormParameters, corpusField, fieldUi);
        }

    }

}
