/* BdfServer - Copyright (c) 2015-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.ficheform;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.ficheform.FicheFormParameters;
import fr.exemole.bdfserver.api.instruction.ProducerParameters;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.api.ui.FieldUi;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.tools.roles.PermissionSummaryBuilder;
import java.text.DecimalFormatSymbols;
import java.util.HashMap;
import java.util.Map;
import net.fichotheque.SubsetItem;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.metadata.CorpusField;
import net.fichotheque.include.LiageTest;
import net.fichotheque.pointeurs.FichePointeur;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.syntax.FicheblockSyntax;
import net.fichotheque.syntax.FormSyntax;
import net.fichotheque.tools.include.LiageTestBuilder;
import net.mapeadores.util.localisation.UserLangContext;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.SeparatorOptions;


/**
 *
 * @author Vincent Calame
 */
public class FicheFormParametersBuilder {

    public final static String PARAM_PREFIX = "param_";
    private final BdfServer bdfServer;
    private final PermissionSummary permissionSummary;
    private final FormSyntax.Parameters ficheItemFormSyntaxParameters;
    private final FicheblockSyntax.Parameters ficheBlockFormSyntaxParameters;
    private final FichePointeur fichePointeur;
    private final Map<String, String> supplementaryParameterMap = new HashMap<String, String>();
    private UserLangContext userLangContext;
    private SubsetItem masterSubsetItem;
    private Redacteur defaultRedacteur;
    private boolean newFiche = false;
    private SeparatorOptions inlineSeparatorOptions = FormSyntax.DEFAULT_INLINE_SEPARATOROPTIONS;
    private SeparatorOptions blockSeparatorOptions = FormSyntax.DEFAULT_BLOCK_SEPARATOROPTIONS;
    private LiageTest liageTest = LiageTestBuilder.ALL_LIAGETEST;

    private FicheFormParametersBuilder(BdfServer bdfServer, FichePointeur fichePointeur, UserLangContext userLangContext, PermissionSummary permissionSummary, FormSyntax.Parameters ficheItemFormSyntaxParameters, FicheblockSyntax.Parameters ficheBlockFormSyntaxParameters) {
        this.bdfServer = bdfServer;
        this.userLangContext = userLangContext;
        this.fichePointeur = fichePointeur;
        this.permissionSummary = permissionSummary;
        this.ficheItemFormSyntaxParameters = ficheItemFormSyntaxParameters;
        this.ficheBlockFormSyntaxParameters = ficheBlockFormSyntaxParameters;
    }

    public FicheFormParametersBuilder setUserLangContext(UserLangContext userLangContext) {
        if (userLangContext == null) {
            throw new IllegalArgumentException("userLangContext is null");
        }
        this.userLangContext = userLangContext;
        ficheItemFormSyntaxParameters.decimalChar((new DecimalFormatSymbols(userLangContext.getFormatLocale())).getDecimalSeparator());
        return this;
    }

    public FicheFormParametersBuilder setNewFiche(boolean newFiche) {
        this.newFiche = newFiche;
        return this;
    }

    public FicheFormParametersBuilder setDefaultRedacteur(Redacteur defaultRedacteur) {
        this.defaultRedacteur = defaultRedacteur;
        return this;
    }

    public FicheFormParametersBuilder setInlineSeparatorOptions(SeparatorOptions inlineSeparatorOptions) {
        if (inlineSeparatorOptions == null) {
            this.inlineSeparatorOptions = FormSyntax.DEFAULT_INLINE_SEPARATOROPTIONS;
        } else {
            this.inlineSeparatorOptions = inlineSeparatorOptions;
        }
        return this;
    }


    public FicheFormParametersBuilder setBlockSeparatorOptions(SeparatorOptions blockSeparatorOptions) {
        if (blockSeparatorOptions == null) {
            this.blockSeparatorOptions = FormSyntax.DEFAULT_BLOCK_SEPARATOROPTIONS;
        } else {
            this.blockSeparatorOptions = blockSeparatorOptions;
        }
        return this;
    }

    public FicheFormParametersBuilder setWithSpecialParseChars(boolean withSpecialParseChars) {
        ficheBlockFormSyntaxParameters.withSpecialParseChars(withSpecialParseChars);
        return this;
    }


    public FicheFormParametersBuilder setMasterSubsetItem(SubsetItem masterSubsetItem) {
        this.masterSubsetItem = masterSubsetItem;
        return this;
    }

    public FicheFormParametersBuilder setLiageTest(LiageTest liageTest) {
        if (liageTest == null) {
            this.liageTest = LiageTestBuilder.ALL_LIAGETEST;
        } else {
            this.liageTest = liageTest;
        }
        return this;
    }

    public FicheFormParametersBuilder initSupplementaryParameter(RequestMap requestMap) {
        String prefix = PARAM_PREFIX;
        for (String name : requestMap.getParameterNameSet()) {
            if (name.startsWith(prefix)) {
                String subname = name.substring(prefix.length());
                String value = requestMap.getParameter(name);
                supplementaryParameterMap.put(subname, value);
            }
        }
        return this;
    }

    public static FicheFormParametersBuilder build(ProducerParameters producerParameters, FichePointeur fichePointeur) {
        return build(producerParameters.getBdfServer(), producerParameters.getBdfUser(), producerParameters.getPermissionSummary(), fichePointeur);
    }

    public static FicheFormParametersBuilder build(BdfServer bdfServer, BdfUser bdfUser, FichePointeur fichePointeur) {
        return build(bdfServer, bdfUser, PermissionSummaryBuilder.build(bdfServer, bdfUser), fichePointeur);
    }

    public static FicheFormParametersBuilder build(BdfServer bdfServer, BdfUser bdfUser, PermissionSummary permissionSummary, FichePointeur fichePointeur) {
        FormSyntax.Parameters ficheItemFormSyntaxParameters = FormSyntax.parameters()
                .defautSphereKey(bdfUser.getRedacteur().getSphere().getSubsetKey())
                .decimalChar((new DecimalFormatSymbols(bdfUser.getFormatLocale())).getDecimalSeparator());
        FicheblockSyntax.Parameters ficheBlockFormSyntaxParameters = FicheblockSyntax.parameters().withSpecialParseChars(true);
        FicheFormParametersBuilder ficheFormParametersBuilder = new FicheFormParametersBuilder(bdfServer, fichePointeur, bdfUser, permissionSummary, ficheItemFormSyntaxParameters, ficheBlockFormSyntaxParameters);
        return ficheFormParametersBuilder
                .setDefaultRedacteur(bdfUser.getRedacteur());
    }


    public FicheFormParameters toFicheFormParameters() {
        return new InternalFicheFormParameters(bdfServer, fichePointeur, userLangContext, permissionSummary,
                ficheItemFormSyntaxParameters, ficheBlockFormSyntaxParameters, supplementaryParameterMap, masterSubsetItem, defaultRedacteur, newFiche,
                inlineSeparatorOptions, blockSeparatorOptions, liageTest);
    }


    private static class InternalFicheFormParameters implements FicheFormParameters {

        private final BdfServer bdfServer;
        private final PermissionSummary permissionSummary;
        private final FormSyntax.Parameters ficheItemFormSyntaxParameters;
        private final FicheblockSyntax.Parameters ficheBlockFormSyntaxParameters;
        private final FichePointeur fichePointeur;
        private final Map<String, String> supplementaryParameterMap;
        private final UserLangContext userLangContext;
        private final SubsetItem masterSubsetItem;
        private final Redacteur defaultRedacteur;
        private final boolean newFiche;
        private final SeparatorOptions inlineSeparatorOptions;
        private final SeparatorOptions blockSeparatorOptions;
        private final LiageTest liageTest;

        private InternalFicheFormParameters(BdfServer bdfServer, FichePointeur fichePointeur, UserLangContext userLangContext, PermissionSummary permissionSummary, FormSyntax.Parameters ficheItemFormSyntaxParameters, FicheblockSyntax.Parameters ficheBlockFormSyntaxParameters,
                Map<String, String> supplementaryParameterMap, SubsetItem masterSubsetItem, Redacteur defaultRedacteur, boolean newFiche,
                SeparatorOptions inlineSeparatorOptions, SeparatorOptions blockSeparatorOptions, LiageTest liageTest) {
            this.bdfServer = bdfServer;
            this.userLangContext = userLangContext;
            this.fichePointeur = fichePointeur;
            this.permissionSummary = permissionSummary;
            this.ficheItemFormSyntaxParameters = ficheItemFormSyntaxParameters;
            this.ficheBlockFormSyntaxParameters = ficheBlockFormSyntaxParameters;
            this.supplementaryParameterMap = supplementaryParameterMap;
            this.masterSubsetItem = masterSubsetItem;
            this.defaultRedacteur = defaultRedacteur;
            this.newFiche = newFiche;
            this.inlineSeparatorOptions = inlineSeparatorOptions;
            this.blockSeparatorOptions = blockSeparatorOptions;
            this.liageTest = liageTest;
        }

        @Override
        public BdfServer getBdfServer() {
            return bdfServer;
        }

        @Override
        public PermissionSummary getPermissionSummary() {
            return permissionSummary;
        }

        @Override
        public UserLangContext getUserLangContext() {
            return userLangContext;
        }

        @Override
        public FichePointeur getFichePointeur() {
            return fichePointeur;
        }

        @Override
        public Corpus getCorpus() {
            return (Corpus) fichePointeur.getSubset();
        }

        @Override
        public CorpusField checkCorpusField(FieldUi fieldUi) {
            return ((Corpus) fichePointeur.getSubset()).getCorpusMetadata().getCorpusField(fieldUi.getFieldKey());
        }

        @Override
        public String getSupplementaryParameter(String name) {
            return supplementaryParameterMap.get(name);
        }

        @Override
        public FormSyntax.Parameters getFicheItemFormSyntaxParameters() {
            return ficheItemFormSyntaxParameters;
        }

        @Override
        public FicheblockSyntax.Parameters getFicheBlockFormSyntaxParameters() {
            return ficheBlockFormSyntaxParameters;
        }

        @Override
        public boolean isNewFiche() {
            return newFiche;
        }

        @Override
        public Redacteur getDefaultRedacteur() {
            return defaultRedacteur;
        }

        @Override
        public SeparatorOptions getBlockSeparatorOptions() {
            return blockSeparatorOptions;
        }

        @Override
        public SubsetItem getMasterSubsetItem() {
            return masterSubsetItem;
        }

        @Override
        public LiageTest getLiageTest() {
            return liageTest;
        }

        @Override
        public SeparatorOptions getInlineSeparatorOptions() {
            return inlineSeparatorOptions;
        }

    }

}
