/* BdfServer - Copyright (c) 2009-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.ficheform.builders;

import fr.exemole.bdfserver.api.ficheform.CorpusIncludeElement;
import fr.exemole.bdfserver.api.ficheform.FicheFormParameters;
import fr.exemole.bdfserver.api.ui.IncludeUi;
import fr.exemole.bdfserver.api.ui.options.LimitOption;
import fr.exemole.bdfserver.tools.L10nUtils;
import fr.exemole.bdfserver.tools.ficheform.FicheFormUtils;
import java.util.AbstractList;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.RandomAccess;
import net.fichotheque.FichothequeConstants;
import net.fichotheque.SubsetItem;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.croisement.Croisements;
import net.fichotheque.croisement.Liaison;
import net.fichotheque.include.ExtendedIncludeKey;
import net.fichotheque.namespaces.SelectSpace;
import net.fichotheque.pointeurs.FichePointeur;
import net.fichotheque.pointeurs.SubsetItemPointeur;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.utils.CroisementUtils;
import net.fichotheque.utils.PointeurUtils;
import net.mapeadores.util.annotation.Nullable;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.attr.Attributes;


/**
 *
 * @author Vincent Calame
 */
public class CorpusIncludeElementBuilder {

    private final String name;
    private final Corpus corpus;
    private final List<InternalEntry> entryList = new ArrayList<InternalEntry>();
    private String label = "";
    private boolean mandatory = false;
    private boolean hasPoidsFilter;
    private int rows = 1;
    private String motcleQueryString;
    private Attributes attributes = AttributeUtils.EMPTY_ATTRIBUTES;

    public CorpusIncludeElementBuilder(String name, Corpus corpus) {
        this.name = name;
        this.corpus = corpus;
    }

    public CorpusIncludeElementBuilder setHasPoidsFilter(boolean hasPoidsFilter) {
        this.hasPoidsFilter = hasPoidsFilter;
        return this;
    }

    public CorpusIncludeElementBuilder setLabel(String label) {
        if (label == null) {
            throw new IllegalArgumentException("label is null");
        }
        this.label = label;
        return this;
    }

    public CorpusIncludeElementBuilder setMandatory(boolean mandatory) {
        this.mandatory = mandatory;
        return this;
    }

    public CorpusIncludeElementBuilder setAttributes(Attributes attributes) {
        if (attributes == null) {
            this.attributes = AttributeUtils.EMPTY_ATTRIBUTES;
        } else {
            this.attributes = attributes;
        }
        return this;
    }

    public CorpusIncludeElementBuilder setRows(int rows) {
        this.rows = rows;
        return this;
    }

    public CorpusIncludeElementBuilder addFicheMeta(FicheMeta ficheMeta, int poids) {
        boolean globalLiage = (name.equals(FichothequeConstants.LIAGE_NAME));
        String value;
        if (globalLiage) {
            value = ficheMeta.getCorpus().getSubsetKey().getSubsetName() + "/" + String.valueOf(ficheMeta.getId());
        } else {
            value = String.valueOf(ficheMeta.getId());
        }
        if (poids > 1) {
            value = value + "<" + poids + ">";
        }
        entryList.add(new InternalEntry(ficheMeta, poids, value));
        return this;
    }

    public CorpusIncludeElementBuilder setMotcleQueryString(String motcleQueryString) {
        this.motcleQueryString = motcleQueryString;
        return this;
    }

    public CorpusIncludeElementBuilder populateOptions(FicheFormParameters ficheFormParameters, IncludeUi includeUi) {
        return this
                .setLabel(L10nUtils.toLabelString(ficheFormParameters, includeUi))
                .setMandatory(FicheFormUtils.isMandatory(includeUi))
                .setAttributes(includeUi.getAttributes());
    }

    public boolean isEmpty() {
        return entryList.isEmpty();
    }

    public CorpusIncludeElement toCorpusIncludeElement() {
        List<CorpusIncludeElement.Entry> finalList = wrap(entryList.toArray(new InternalEntry[entryList.size()]));
        return new InternalCorpusIncludeElement(name, corpus, hasPoidsFilter, label, mandatory, attributes, finalList, rows, motcleQueryString);
    }

    @Nullable
    public static CorpusIncludeElementBuilder check(FicheFormParameters ficheFormParameters, IncludeUi includeUi) {
        ExtendedIncludeKey includeKey = includeUi.getExtendedIncludeKey();
        CorpusIncludeElementBuilder builder;
        if (includeKey == null) {
            if (!includeUi.getName().equals(FichothequeConstants.LIAGE_NAME)) {
                return null;
            }
            builder = init(FichothequeConstants.LIAGE_NAME, null)
                    .setHasPoidsFilter(false)
                    .setRows(4);
            if (!ficheFormParameters.isNewFiche()) {
                FichePointeur fichePointeur = ficheFormParameters.getFichePointeur();
                Collection<Liaison> liaisons = CroisementUtils.getNoList(fichePointeur, "", ficheFormParameters.getLiageTest(), ficheFormParameters.getPermissionSummary().getSubsetAccessPredicate(), null);
                for (Liaison liaison : liaisons) {
                    builder.addFicheMeta((FicheMeta) liaison.getSubsetItem(), liaison.getLien().getPoids());
                }
            }
            LimitOption limitOption = includeUi.getLimitOption();
            boolean noNew = limitOption.isNoNewAllowed();
            if (noNew) {
                if (ficheFormParameters.isNewFiche()) {
                    return null;
                } else {
                    if (builder.isEmpty()) {
                        return null;
                    }
                }
            }
        } else {
            SubsetKey subsetKey = includeKey.getSubsetKey();
            if (!subsetKey.isCorpusSubset()) {
                return null;
            }
            Corpus corpus = (Corpus) ficheFormParameters.getBdfServer().getFichotheque().getSubset(subsetKey);
            if (corpus == null) {
                return null;
            }
            if (!FicheFormUtils.hasAccess(ficheFormParameters, includeUi)) {
                return null;
            }
            builder = CorpusIncludeElementBuilder.init(includeUi.getName(), corpus)
                    .setHasPoidsFilter(includeKey.hasPoidsFilter());
            if (!ficheFormParameters.isNewFiche()) {
                SubsetItemPointeur pointeur = PointeurUtils.checkMasterPointeur(ficheFormParameters.getFichePointeur(), includeKey.isMaster());
                populate(builder, corpus, pointeur, includeKey);
            } else if (includeKey.isMaster()) {
                SubsetItem masterSubsetItem = ficheFormParameters.getMasterSubsetItem();
                if ((masterSubsetItem != null) && (masterSubsetItem instanceof Motcle)) {
                    SubsetItemPointeur pointeur = ficheFormParameters.getFichePointeur().getRattachementPointeur(masterSubsetItem.getSubset().getSubsetKey());
                    pointeur.setCurrentSubsetItem(masterSubsetItem);
                    populate(builder, corpus, pointeur, includeKey);
                }
            }
        }
        Attribute motcleSelect = includeUi.getAttributes().getAttribute(SelectSpace.MOTCLE_KEY);
        String motcleQueryString = null;
        if (motcleSelect != null) {
            motcleQueryString = FicheFormUtils.replaceGenericParameters(ficheFormParameters, motcleSelect.getFirstValue());
        }
        return builder
                .populateOptions(ficheFormParameters, includeUi)
                .setMotcleQueryString(motcleQueryString);
    }

    private static void populate(CorpusIncludeElementBuilder builder, Corpus corpus, SubsetItemPointeur pointeur, ExtendedIncludeKey includeKey) {
        Croisements ficheCroisements = pointeur.getCroisements(corpus);
        Collection<Liaison> liaisons = CroisementUtils.filter(ficheCroisements, includeKey.getMode(), includeKey.getPoidsFilter());
        if (includeKey.hasPoidsFilter()) {
            for (Liaison liaison : liaisons) {
                builder.addFicheMeta((FicheMeta) liaison.getSubsetItem(), -1);
            }
        } else {
            for (Liaison liaison : liaisons) {
                builder.addFicheMeta((FicheMeta) liaison.getSubsetItem(), liaison.getLien().getPoids());
            }
        }
    }

    public static CorpusIncludeElementBuilder init(String name, Corpus corpus) {
        return new CorpusIncludeElementBuilder(name, corpus);
    }

    private static List<CorpusIncludeElement.Entry> wrap(CorpusIncludeElement.Entry[] array) {
        return new EntryList(array);
    }


    private static class InternalCorpusIncludeElement implements CorpusIncludeElement {

        private final String name;
        private final Corpus corpus;
        private final boolean withPoidsFilter;
        private final String label;
        private final boolean mandatory;
        private final Attributes attributes;
        private final List<CorpusIncludeElement.Entry> entryList;
        private final int rows;
        private final String motcleQueryString;


        private InternalCorpusIncludeElement(String name, Corpus corpus, boolean withPoidsFilter, String label, boolean mandatory, Attributes attributes, List<CorpusIncludeElement.Entry> entryList, int rows, String motcleQueryString) {
            this.name = name;
            this.withPoidsFilter = withPoidsFilter;
            this.corpus = corpus;
            this.label = label;
            this.mandatory = mandatory;
            this.attributes = attributes;
            this.rows = rows;
            this.entryList = entryList;
            this.motcleQueryString = motcleQueryString;
        }

        @Override
        public String getIncludeName() {
            return name;
        }

        @Override
        public Corpus getCorpus() {
            return corpus;
        }

        @Override
        public boolean hasPoidsFilter() {
            return withPoidsFilter;
        }

        @Override
        public String getLabel() {
            return label;
        }

        @Override
        public boolean isMandatory() {
            return mandatory;
        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

        @Override
        public List<Entry> getEntryList() {
            return entryList;
        }

        @Override
        public int getRows() {
            return rows;
        }

        @Override
        public String getMotcleQueryString() {
            return motcleQueryString;
        }

    }


    private static class InternalEntry implements CorpusIncludeElement.Entry {

        private final FicheMeta ficheMeta;
        private final int poids;
        private final String value;

        private InternalEntry(FicheMeta ficheMeta, int poids, String value) {
            this.ficheMeta = ficheMeta;
            this.poids = poids;
            this.value = value;
        }

        @Override
        public FicheMeta getFicheMeta() {
            return ficheMeta;
        }

        @Override
        public int getPoids() {
            return poids;
        }

        @Override
        public String getValue() {
            return value;
        }

    }


    private static class EntryList extends AbstractList<CorpusIncludeElement.Entry> implements RandomAccess {

        private final CorpusIncludeElement.Entry[] array;

        private EntryList(CorpusIncludeElement.Entry[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public CorpusIncludeElement.Entry get(int index) {
            return array[index];
        }

    }

}
