/* BdfServer - Copyright (c) 2020-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.ficheform.builders;

import fr.exemole.bdfserver.api.ficheform.FicheFormParameters;
import fr.exemole.bdfserver.api.ficheform.ThesaurusIncludeElement;
import fr.exemole.bdfserver.api.ui.IncludeUi;
import fr.exemole.bdfserver.tools.L10nUtils;
import fr.exemole.bdfserver.tools.externalsource.ExternalSourceUtils;
import fr.exemole.bdfserver.tools.ficheform.FicheFormUtils;
import java.util.AbstractList;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.RandomAccess;
import net.fichotheque.croisement.Croisements;
import net.fichotheque.croisement.Liaison;
import net.fichotheque.include.ExtendedIncludeKey;
import net.fichotheque.namespaces.BdfSpace;
import net.fichotheque.pointeurs.SubsetItemPointeur;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.utils.CroisementUtils;
import net.mapeadores.util.annotation.Nullable;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.attr.Attributes;


/**
 *
 * @author Vincent Calame
 */
public class FicheStyleThesaurusIncludeElementBuilder {

    private final String name;
    private final Thesaurus thesaurus;
    private final List<InternalEntry> entryList = new ArrayList<InternalEntry>();
    private String label = "";
    private boolean mandatory = false;
    private boolean hasPoidsFilter;
    private Attribute idalphaStyle = null;
    private boolean withExternalSource;
    private Attributes attributes = AttributeUtils.EMPTY_ATTRIBUTES;

    public FicheStyleThesaurusIncludeElementBuilder(String name, Thesaurus thesaurus) {
        this.name = name;
        this.thesaurus = thesaurus;
    }

    public FicheStyleThesaurusIncludeElementBuilder setLabel(String label) {
        if (label == null) {
            throw new IllegalArgumentException("label is null");
        }
        this.label = label;
        return this;
    }

    public FicheStyleThesaurusIncludeElementBuilder setMandatory(boolean mandatory) {
        this.mandatory = mandatory;
        return this;
    }

    public FicheStyleThesaurusIncludeElementBuilder setAttributes(Attributes attributes) {
        if (attributes == null) {
            this.attributes = AttributeUtils.EMPTY_ATTRIBUTES;
        } else {
            this.attributes = attributes;
        }
        return this;
    }

    public FicheStyleThesaurusIncludeElementBuilder setIdalphaStyle(Attribute idalphaStyle) {
        this.idalphaStyle = idalphaStyle;
        return this;
    }

    public FicheStyleThesaurusIncludeElementBuilder setHasPoidsFilter(boolean hasPoidsFilter) {
        this.hasPoidsFilter = hasPoidsFilter;
        return this;
    }

    public FicheStyleThesaurusIncludeElementBuilder addMotcle(Motcle motcle, int poids) {
        String idalpha = motcle.getIdalpha();
        String value;
        if (idalpha != null) {
            value = idalpha;
        } else {
            value = String.valueOf(motcle.getId());
        }
        if (poids > 1) {
            value = value + "<" + poids + ">";
        }
        entryList.add(new InternalEntry(motcle, poids, value));
        return this;
    }

    public FicheStyleThesaurusIncludeElementBuilder setWithExternalSource(boolean withExternalSource) {
        this.withExternalSource = withExternalSource;
        return this;
    }

    public ThesaurusIncludeElement.FicheStyle toFicheStyleThesaurusIncludeElement() {
        List<ThesaurusIncludeElement.Entry> finalList = wrap(entryList.toArray(new InternalEntry[entryList.size()]));
        return new InternalFicheStyleThesaurusIncludeElement(name, label, mandatory, attributes, thesaurus, idalphaStyle, hasPoidsFilter, finalList, withExternalSource);
    }

    @Nullable
    public static FicheStyleThesaurusIncludeElementBuilder check(FicheFormParameters ficheFormParameters, IncludeUi includeUi) {
        ExtendedIncludeKey includeKey = includeUi.getExtendedIncludeKey();
        if (includeKey == null) {
            return null;
        }
        Thesaurus thesaurus = (Thesaurus) ficheFormParameters.getBdfServer().getFichotheque().getSubset(includeKey.getSubsetKey());
        if (thesaurus == null) {
            return null;
        }
        if (!FicheFormUtils.hasAccess(ficheFormParameters, includeUi)) {
            return null;
        }
        boolean isMandatory = FicheFormUtils.isMandatory(includeUi);
        Attribute idalphaStyle = includeUi.getAttributes().getAttribute(BdfSpace.IDALPHASTYLE_KEY);
        if (idalphaStyle == null) {
            idalphaStyle = thesaurus.getThesaurusMetadata().getAttributes().getAttribute(BdfSpace.IDALPHASTYLE_KEY);
        }
        boolean withExternalSource = (ExternalSourceUtils.getExternalSource(ficheFormParameters.getBdfServer(), thesaurus) != null);
        FicheStyleThesaurusIncludeElementBuilder builder = init(includeUi.getName(), thesaurus)
                .setMandatory(isMandatory)
                .setAttributes(includeUi.getAttributes())
                .setIdalphaStyle(idalphaStyle)
                .setLabel(L10nUtils.toLabelString(ficheFormParameters, includeUi))
                .setHasPoidsFilter(includeKey.hasPoidsFilter())
                .setWithExternalSource(withExternalSource);
        if (!ficheFormParameters.isNewFiche()) {
            populate(builder, thesaurus, ficheFormParameters.getFichePointeur(), includeKey);
        }
        return builder;
    }

    public static FicheStyleThesaurusIncludeElementBuilder init(String name, Thesaurus thesaurus) {
        return new FicheStyleThesaurusIncludeElementBuilder(name, thesaurus);
    }

    private static void populate(FicheStyleThesaurusIncludeElementBuilder builder, Thesaurus thesaurus, SubsetItemPointeur pointeur, ExtendedIncludeKey includeKey) {
        Croisements ficheCroisements = pointeur.getCroisements(thesaurus);
        Collection<Liaison> liaisons = CroisementUtils.filter(ficheCroisements, includeKey.getMode(), includeKey.getPoidsFilter());
        if (includeKey.hasPoidsFilter()) {
            for (Liaison liaison : liaisons) {
                builder.addMotcle((Motcle) liaison.getSubsetItem(), -1);
            }
        } else {
            for (Liaison liaison : liaisons) {
                builder.addMotcle((Motcle) liaison.getSubsetItem(), liaison.getLien().getPoids());
            }
        }
    }

    private static List<ThesaurusIncludeElement.Entry> wrap(ThesaurusIncludeElement.Entry[] array) {
        return new EntryList(array);
    }


    private static class InternalFicheStyleThesaurusIncludeElement implements ThesaurusIncludeElement.FicheStyle {

        private final String name;
        private final String label;
        private final boolean mandatory;
        private final Attributes attributes;
        private final Thesaurus thesaurus;
        private final Attribute idalphaStyle;
        private final boolean withPoidsFilter;
        private final java.util.List<ThesaurusIncludeElement.Entry> list;
        private final boolean withExternalSource;

        private InternalFicheStyleThesaurusIncludeElement(String name, String label, boolean mandatory, Attributes attributes, Thesaurus thesaurus, Attribute idalphaStyle, boolean withPoidsFilter, java.util.List<ThesaurusIncludeElement.Entry> list, boolean withExternalSource) {
            this.name = name;
            this.label = label;
            this.mandatory = mandatory;
            this.attributes = attributes;
            this.thesaurus = thesaurus;
            this.idalphaStyle = idalphaStyle;
            this.withPoidsFilter = withPoidsFilter;
            this.list = list;
            this.withExternalSource = withExternalSource;
        }

        @Override
        public String getIncludeName() {
            return name;
        }

        @Override
        public String getLabel() {
            return label;
        }

        @Override
        public boolean isMandatory() {
            return mandatory;
        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

        @Override
        public Thesaurus getThesaurus() {
            return thesaurus;
        }

        @Override
        public Attribute getIdalphaStyle() {
            return idalphaStyle;
        }

        @Override
        public boolean hasPoidsFilter() {
            return withPoidsFilter;
        }

        @Override
        public java.util.List<Entry> getEntryList() {
            return list;
        }

        @Override
        public boolean isWithExternalSource() {
            return withExternalSource;
        }

    }


    private static class InternalEntry implements ThesaurusIncludeElement.Entry {

        private final Motcle motcle;
        private final int poids;
        private final String value;

        private InternalEntry(Motcle motcle, int poids, String value) {
            this.motcle = motcle;
            this.poids = poids;
            this.value = value;
        }

        @Override
        public Motcle getMotcle() {
            return motcle;
        }

        @Override
        public int getPoids() {
            return poids;
        }

        @Override
        public String getValue() {
            return value;
        }

    }


    private static class EntryList extends AbstractList<ThesaurusIncludeElement.Entry> implements RandomAccess {

        private final ThesaurusIncludeElement.Entry[] array;

        private EntryList(ThesaurusIncludeElement.Entry[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public ThesaurusIncludeElement.Entry get(int index) {
            return array[index];
        }

    }

}
