/* BdfServer - Copyright (c) 2015-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.ui.components;

import fr.exemole.bdfserver.api.ui.CommentUi;
import fr.exemole.bdfserver.api.ui.FieldUi;
import fr.exemole.bdfserver.api.ui.IncludeUi;
import fr.exemole.bdfserver.api.ui.UiComponent;
import fr.exemole.bdfserver.api.ui.UiComponents;
import fr.exemole.bdfserver.api.ui.options.InputOption;
import fr.exemole.bdfserver.api.ui.options.LimitOption;
import fr.exemole.bdfserver.api.ui.options.ValueOption;
import fr.exemole.bdfserver.tools.ui.UiUtils;
import fr.exemole.bdfserver.tools.ui.options.UiOptionsUtils;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.include.ExtendedIncludeKey;
import net.fichotheque.include.IncludeKey;
import net.fichotheque.namespaces.SelectSpace;
import net.fichotheque.utils.FichothequeUtils;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.html.TrustedHtmlFactory;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelUtils;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.DomMessages;
import net.mapeadores.util.xml.ElementHandler;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public final class UiComponentUtils {

    private UiComponentUtils() {

    }

    public static List<CommentUi> getCommentUiList(UiComponents uiComponents) {
        List<UiComponent> uiComponentList = uiComponents.getUiComponentList();
        if (uiComponentList.isEmpty()) {
            return Collections.emptyList();
        }
        List<CommentUi> commentUiList = new ArrayList<CommentUi>();
        Set<String> doneSet = new HashSet<String>();
        for (UiComponent uiComponent : uiComponentList) {
            if (uiComponent instanceof CommentUi) {
                CommentUi commentUi = (CommentUi) uiComponent;
                if (!doneSet.contains(commentUi.getName())) {
                    doneSet.add(commentUi.getName());
                    commentUiList.add(commentUi);
                }
            }
        }
        return commentUiList;
    }

    public static String[] toPositionArray(UiComponents uiComponents) {
        List<UiComponent> uiComponentList = uiComponents.getUiComponentList();
        int size = uiComponentList.size();
        String[] result = new String[size];
        for (int i = 0; i < size; i++) {
            result[i] = uiComponentList.get(i).getName();
        }
        return result;
    }

    public static UiSummary summarize(UiComponents uiComponents) {
        boolean addendaInclude = false;
        boolean albumInclude = false;
        for (UiComponent uiComponent : uiComponents.getUiComponentList()) {
            if (uiComponent instanceof IncludeUi) {
                IncludeUi includeUi = (IncludeUi) uiComponent;
                switch (UiUtils.getSubsetCategory(includeUi)) {
                    case SubsetKey.CATEGORY_ADDENDA:
                        addendaInclude = true;
                        break;
                    case SubsetKey.CATEGORY_ALBUM:
                        albumInclude = true;
                        break;
                }
            }
        }
        return new UiSummary(addendaInclude, albumInclude);
    }

    public static CommentUi readCommentUi(Element element_xml, TrustedHtmlFactory trustedHtmlFactory, MessageHandler messageHandler, String xpath) {
        String name = element_xml.getAttribute("name");
        if (name.length() == 0) {
            DomMessages.emptyAttribute(messageHandler, xpath, "name");
            return null;
        }
        CommentUiBuilder commentUiBuilder = CommentUiBuilder.init(trustedHtmlFactory, name);
        String locationString = element_xml.getAttribute("location");
        if (locationString.length() != 0) {
            int location = UiUtils.maskStringToInt(locationString);
            commentUiBuilder.setLocation(location);
        }
        DOMUtils.readChildren(element_xml, new CommentUiHandler(commentUiBuilder, messageHandler));
        return commentUiBuilder.toCommentUi();
    }

    public static FieldUi readFieldUi(Element element_xml, MessageHandler messageHandler, String xpath) {
        String fieldKeyString = element_xml.getAttribute("field-key");
        if (fieldKeyString.length() == 0) {
            DomMessages.emptyAttribute(messageHandler, xpath, "field-key");
            return null;
        }
        FieldKey fieldKey;
        try {
            fieldKey = FieldKey.parse(fieldKeyString);
        } catch (ParseException pe) {
            DomMessages.wrongAttributeValue(messageHandler, xpath, "field-key", fieldKeyString);
            return null;
        }
        FieldUiBuilder fieldUiBuilder = FieldUiBuilder.init(fieldKey);
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("input")) {
                    InputOption inputOption = UiOptionsUtils.readInputOption(element, messageHandler);
                    if (inputOption != null) {
                        fieldUiBuilder.setInputOption(inputOption);
                    }
                } else if (tagName.equals("value")) {
                    ValueOption valueOption = UiOptionsUtils.readValueOption(element, messageHandler);
                    if (valueOption != null) {
                        fieldUiBuilder.setValueOption(valueOption);
                    }
                } else if (tagName.equals("attr")) {
                    AttributeUtils.readAttrElement(fieldUiBuilder.getAttributesBuilder(), element);
                }
            }
        }
        return fieldUiBuilder.toFieldUi();
    }

    public static IncludeUi readSubsetIncludeUi(Element element_xml, MessageHandler messageHandler, String xpath) {
        boolean master = false;
        SubsetKey subsetKey = null;
        String mode = "";
        int poids = -1;
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("master")) {
                    master = true;
                } else if (tagName.equals("subset")) {
                    String subsetKeyString = XMLUtils.getData(element);
                    if (subsetKeyString.length() == 0) {
                        DomMessages.emptyElement(messageHandler, "subset");
                        return null;
                    }
                    try {
                        subsetKey = SubsetKey.parse(subsetKeyString);
                    } catch (java.text.ParseException pe) {
                        DomMessages.wrongElementValue(messageHandler, "subset", subsetKeyString);
                        return null;
                    }
                } else if (tagName.equals("poids")) {
                    String poidsString = XMLUtils.getData(element);
                    if (poidsString.length() == 0) {
                        DomMessages.emptyElement(messageHandler, "poids");
                        return null;
                    }
                    try {
                        poids = Integer.parseInt(poidsString);
                    } catch (NumberFormatException nfe) {
                        DomMessages.wrongElementValue(messageHandler, "poids", poidsString);
                        return null;
                    }
                } else if (tagName.equals("mode")) {
                    mode = XMLUtils.getData(element);
                    if (mode.length() > 0) {
                        try {
                            StringUtils.checkTechnicalName(mode, false);
                        } catch (ParseException pe) {
                            DomMessages.wrongElementValue(messageHandler, "mode", mode);
                            return null;
                        }
                    }
                }
            }
        }
        if (subsetKey == null) {
            DomMessages.emptyElement(messageHandler, xpath);
            return null;
        }
        try {
            ExtendedIncludeKey extendedIncludeKey = ExtendedIncludeKey.newInstance(IncludeKey.newInstance(subsetKey, mode, poids), master);
            return readIncludeUi(IncludeUiBuilder.init(extendedIncludeKey), element_xml, messageHandler, false);
        } catch (IllegalArgumentException iae) {
            DomMessages.wrongElementValue(messageHandler, xpath, subsetKey.toString());
            return null;
        }
    }

    public static IncludeUi readSpecialIncludeUi(Element element_xml, MessageHandler messageHandler, String xpath) {
        String includeKeyString = element_xml.getAttribute("include-key");
        if (includeKeyString.length() == 0) {
            DomMessages.emptyAttribute(messageHandler, xpath, "include-key");
            return null;
        }
        boolean ignoreLimit;
        IncludeUiBuilder includeUiBuilder;
        if (FichothequeUtils.isSpecialIncludeName(includeKeyString)) {
            includeUiBuilder = IncludeUiBuilder.init(includeKeyString);
            ignoreLimit = false;
        } else if (includeKeyString.equals("association")) {
            ExtendedIncludeKey extendedIncludeKey = ExtendedIncludeKey.build("association");
            includeUiBuilder = IncludeUiBuilder.init(extendedIncludeKey);
            ignoreLimit = true;
        } else {
            DomMessages.wrongAttributeValue(messageHandler, xpath, "include-key", includeKeyString);
            return null;
        }
        return readIncludeUi(includeUiBuilder, element_xml, messageHandler, ignoreLimit);
    }

    private static IncludeUi readIncludeUi(IncludeUiBuilder includeUiBuilder, Element element_xml, MessageHandler messageHandler, boolean ignoreLimit) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("custom")) {
                    readCustom(includeUiBuilder, element, messageHandler);
                }
                if (tagName.equals("input")) {
                    InputOption inputOption = UiOptionsUtils.readInputOption(element, messageHandler);
                    if (inputOption != null) {
                        includeUiBuilder.setInputOption(inputOption);
                    }
                } else if (tagName.equals("value")) {
                    ValueOption valueOption = UiOptionsUtils.readValueOption(element, messageHandler);
                    if (valueOption != null) {
                        includeUiBuilder.setValueOption(valueOption);
                    }
                } else if (tagName.equals("limit")) {
                    LimitOption limitOption = UiOptionsUtils.readLimitOption(element, messageHandler);
                    if ((limitOption != null) && (!ignoreLimit)) {
                        includeUiBuilder.setLimitOption(limitOption);
                    }
                } else if (tagName.equals("pioche-select")) {
                    String motcleQueryString = XMLUtils.getData(element);
                    if (motcleQueryString.length() > 0) {
                        includeUiBuilder.getAttributesBuilder().appendValue(SelectSpace.MOTCLE_KEY, motcleQueryString);
                    }
                } else if (tagName.equals("attr")) {
                    AttributeUtils.readAttrElement(includeUiBuilder.getAttributesBuilder(), element);
                }
            }
        }
        return includeUiBuilder.toIncludeUi();
    }

    private static void readCustom(IncludeUiBuilder includeUiBuilder, Element element_xml, MessageHandler messageHandler) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if ((tagName.equals("label")) || (tagName.equals("lib"))) {
                    try {
                        Label label = LabelUtils.readLabel(element);
                        if (label != null) {
                            includeUiBuilder.putLabel(label);
                        }
                    } catch (ParseException ile) {
                        DomMessages.wrongLangAttribute(messageHandler, tagName, element.getAttribute("xml:lang"));
                    }
                } else {
                    DomMessages.unknownTagWarning(messageHandler, tagName);
                }
            }
        }
    }


    private static class CommentUiHandler implements ElementHandler {

        private final CommentUiBuilder commentUiBuilder;
        private final MessageHandler messageHandler;

        private CommentUiHandler(CommentUiBuilder commentUiBuilder, MessageHandler messageHandler) {
            this.commentUiBuilder = commentUiBuilder;
            this.messageHandler = messageHandler;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if ((tagName.equals("html")) || (tagName.equals("lib"))) {
                String langString = element.getAttribute("xml:lang");
                if (langString.length() > 0) {
                    try {
                        Lang lang = Lang.parse(langString);
                        String htmlString = XMLUtils.getRawData(element);
                        commentUiBuilder.putHtml(lang, htmlString);
                    } catch (ParseException pe) {
                        DomMessages.wrongLangAttribute(messageHandler, tagName, langString);
                    }
                } else {
                    DomMessages.emptyAttribute(messageHandler, tagName, "xml:lang");
                }
            } else if (tagName.equals("attr")) {
                AttributeUtils.readAttrElement(commentUiBuilder.getAttributesBuilder(), element);
            } else {
                DomMessages.unknownTagWarning(messageHandler, tagName);
            }
        }

    }

}
