/* BdfServer - Copyright (c) 2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.uml;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.managers.UiManager;
import fr.exemole.bdfserver.api.subsettree.GroupNode;
import fr.exemole.bdfserver.api.subsettree.SubsetNode;
import fr.exemole.bdfserver.api.subsettree.SubsetTree;
import fr.exemole.bdfserver.api.ui.IncludeUi;
import fr.exemole.bdfserver.api.ui.UiComponent;
import fr.exemole.bdfserver.api.ui.UiComponents;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;
import net.fichotheque.Fichotheque;
import net.fichotheque.Subset;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.include.ExtendedIncludeKey;
import net.fichotheque.include.IncludeKey;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.utils.FichothequeMetadataUtils;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.MessageLocalisation;
import net.mapeadores.util.uml.PumlUtils;
import net.mapeadores.util.uml.UmlClass;


/**
 *
 * @author Vincent Calame
 */
public class FichothequePumlWriter {

    private final BdfServer bdfServer;
    private final Lang lang;
    private final MessageLocalisation messageLocalisation;
    private final SortedMap<SubsetKey, SubsetInfo> subsetInfoMap = new TreeMap<SubsetKey, SubsetInfo>();
    private final List<SubsetInfo> thesaurusList = new ArrayList<SubsetInfo>();
    private final List<CorpusInfo> corpusList = new ArrayList<CorpusInfo>();
    private final List<CorpusInfo> satelliteCorpusList = new ArrayList<CorpusInfo>();
    private final List<SubsetInfo> otherList = new ArrayList<SubsetInfo>();

    public FichothequePumlWriter(BdfServer bdfServer, Lang lang, MessageLocalisation messageLocalisation) {
        this.bdfServer = bdfServer;
        this.lang = lang;
        this.messageLocalisation = messageLocalisation;
    }

    public void write(Appendable appendable) throws IOException {
        scanTree(SubsetKey.CATEGORY_THESAURUS);
        scanTree(SubsetKey.CATEGORY_CORPUS);
        scanTree(SubsetKey.CATEGORY_ADDENDA);
        scanTree(SubsetKey.CATEGORY_ALBUM);
        scanRelations(corpusList);
        scanRelations(satelliteCorpusList);
        Fichotheque fichotheque = bdfServer.getFichotheque();
        PumlUtils.start(appendable);
        PumlUtils.title(appendable, FichothequeMetadataUtils.getTitle(fichotheque, lang));
        PumlUtils.hideEmpties(appendable);
        PumlUtils.orthoLineType(appendable);
        writeClasses(appendable, thesaurusList);
        writeClasses(appendable, corpusList);
        writeClasses(appendable, satelliteCorpusList);
        writeClasses(appendable, otherList);
        writeMaster(appendable, corpusList);
        writeMaster(appendable, satelliteCorpusList);
        writeRelations(appendable, thesaurusList, ">", true);
        writeRelations(appendable, corpusList);
        writeRelations(appendable, satelliteCorpusList);
        writeRelations(appendable, otherList, "o", false);
        writeHidden(appendable, thesaurusList, ">", true);
        writeHidden(appendable, corpusList);
        writeHidden(appendable, otherList, "o", false);
        PumlUtils.end(appendable);
    }

    private void writeClasses(Appendable appendable, List infoList) throws IOException {
        if (infoList.isEmpty()) {
            return;
        }
        for (Object subsetInfo : infoList) {
            ((SubsetInfo) subsetInfo).umlClass.write(appendable, 0);
        }
        appendable.append("\n");
    }

    private void writeMaster(Appendable appendable, List<CorpusInfo> infoList) throws IOException {
        for (CorpusInfo corpusInfo : infoList) {
            Subset masterSubset = corpusInfo.getCorpus().getMasterSubset();
            if (masterSubset != null) {
                appendable.append(masterSubset.getSubsetKey().toString());
                appendable.append(" *-- ");
                appendable.append(corpusInfo.getSubsetKey().toString());
                appendable.append("\n");
            }
        }
    }

    private void writeRelations(Appendable appendable, List<SubsetInfo> infoList, String relationType, boolean top) throws IOException {
        for (SubsetInfo subsetInfo : infoList) {
            SubsetKey currentKey = subsetInfo.getSubsetKey();
            for (IncludeKey includeKey : subsetInfo.getRelationSet()) {
                if (top) {
                    appendable.append(currentKey.toString());
                    appendable.append(" --");
                    appendable.append(relationType);
                    appendable.append(" ");
                    appendable.append(includeKey.getSubsetKey().toString());

                } else {
                    appendable.append(includeKey.getSubsetKey().toString());
                    appendable.append(" ");
                    appendable.append(relationType);
                    appendable.append("-- ");
                    appendable.append(currentKey.toString());
                }
                String mode = includeKey.getMode();
                if (!mode.isEmpty()) {
                    appendable.append(" : ");
                    appendable.append(mode);
                }
                appendable.append("\n");
            }
        }
    }

    private void writeRelations(Appendable appendable, List<CorpusInfo> infoList) throws IOException {
        for (CorpusInfo corpusInfo : infoList) {
            SubsetKey currentKey = corpusInfo.getSubsetKey();
            for (IncludeKey includeKey : corpusInfo.getRelationSet()) {
                String direction = "right";
                if (!corpusInfo.isSatellite) {
                    CorpusInfo otherInfo = (CorpusInfo) subsetInfoMap.get(includeKey.getSubsetKey());
                    if (otherInfo.isSatellite) {
                        direction = "down";
                    }
                }
                appendable.append(currentKey.toString());
                appendable.append(" -");
                appendable.append(direction);
                appendable.append("- ");
                appendable.append(includeKey.getSubsetKey().toString());
                appendable.append("\n");
            }
        }
    }

    private void writeHidden(Appendable appendable, List<SubsetInfo> infoList, String relationType, boolean top) throws IOException {
        for (SubsetInfo subsetInfo : infoList) {
            SubsetKey currentKey = subsetInfo.getSubsetKey();
            if (subsetInfo.getRelationSet().isEmpty()) {
                CorpusInfo corpusInfo = getNearestCorpusSubsetInfo(subsetInfo.index);
                if (corpusInfo != null) {
                    if (top) {
                        appendable.append(currentKey.toString());
                        appendable.append(" -[hidden]-");
                        appendable.append(relationType);
                        appendable.append(" ");
                        appendable.append(corpusInfo.getSubsetKey().toString());

                    } else {
                        appendable.append(corpusInfo.getSubsetKey().toString());
                        appendable.append(" ");
                        appendable.append(relationType);
                        appendable.append("-[hidden]- ");
                        appendable.append(currentKey.toString());
                    }
                    if (currentKey.isThesaurusSubset()) {
                        corpusInfo.setWithThesaurusRelation(true);
                    }
                }
                appendable.append("\n");
            }
        }
    }

    private void writeHidden(Appendable appendable, List<CorpusInfo> infoList) throws IOException {
        SubsetKey thesaurusKey = getPreferredThesaurusKeyForHidden();
        for (CorpusInfo corpusInfo : infoList) {
            if ((!corpusInfo.isWithThesaurusRelation()) && (thesaurusKey != null)) {
                appendable.append(thesaurusKey.toString());
                appendable.append(" -[hidden]->");
                appendable.append(corpusInfo.getSubsetKey().toString());
                appendable.append("\n");
            }
        }
    }

    private SubsetKey getPreferredThesaurusKeyForHidden() {
        int size = thesaurusList.size();
        if (size == 0) {
            return null;
        }
        for (int i = (size - 1); i >= 0; i--) {
            SubsetInfo subsetInfo = thesaurusList.get(i);
            if (subsetInfo.getRelationSet().isEmpty()) {
                return subsetInfo.getSubsetKey();
            }
        }
        return thesaurusList.get(size - 1).getSubsetKey();
    }


    private CorpusInfo getNearestCorpusSubsetInfo(int index) {
        int size = corpusList.size();
        if (size == 0) {
            return null;
        }
        if (index < size) {
            return corpusList.get(index);
        } else {
            return corpusList.get(size - 1);
        }
    }

    private void scanRelations(List<CorpusInfo> corpusInfoList) {
        UiManager uiManager = bdfServer.getUiManager();
        for (CorpusInfo corpusInfo : corpusInfoList) {
            Corpus corpus = corpusInfo.getCorpus();
            SubsetKey currentCorpusKey = corpusInfo.getSubsetKey();
            SubsetInfo masterSubsetInfo = null;
            Subset masterSubset = corpus.getMasterSubset();
            if (masterSubset != null) {
                masterSubsetInfo = subsetInfoMap.get(masterSubset.getSubsetKey());
            }
            UiComponents uiComponents = uiManager.getMainUiComponents(corpus);
            for (UiComponent uiComponent : uiComponents.getUiComponentList()) {
                if (uiComponent instanceof IncludeUi) {
                    IncludeUi includeUi = (IncludeUi) uiComponent;
                    ExtendedIncludeKey eik = includeUi.getExtendedIncludeKey();
                    if (eik != null) {
                        SubsetKey otherKey = eik.getSubsetKey();
                        SubsetInfo otherInfo = subsetInfoMap.get(otherKey);
                        if (otherInfo != null) {
                            if (eik.isMaster()) {
                                if ((masterSubsetInfo != null) && (otherKey.isCorpusSubset())) {
                                    SubsetKey masterSubsetKey = masterSubsetInfo.getSubsetKey();
                                    if (masterSubsetKey.isThesaurusSubset()) {
                                        masterSubsetInfo.addIncludeKey(IncludeKey.newInstance(otherKey, eik.getMode(), -1));
                                        ((CorpusInfo) otherInfo).setWithThesaurusRelation(true);
                                    }
                                }
                            } else {
                                if (otherKey.isCorpusSubset()) {
                                    addRelation(corpusInfo, (CorpusInfo) otherInfo, eik.getMode());
                                } else {
                                    otherInfo.addIncludeKey(IncludeKey.newInstance(currentCorpusKey, eik.getMode(), -1));
                                    if (otherKey.isThesaurusSubset()) {
                                        corpusInfo.setWithThesaurusRelation(true);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    private void addRelation(CorpusInfo corpusInfo1, CorpusInfo corpusInfo2, String mode) {
        if (!corpusInfo1.isBefore(corpusInfo2)) {
            CorpusInfo tmp = corpusInfo2;
            corpusInfo2 = corpusInfo1;
            corpusInfo1 = tmp;
        }
        corpusInfo1.addIncludeKey(IncludeKey.newInstance(corpusInfo2.getSubsetKey(), mode, -1));
    }

    private void scanTree(short category) {
        SubsetTree subsetTree = bdfServer.getTreeManager().getSubsetTree(category);
        for (SubsetTree.Node node : subsetTree.getNodeList()) {
            if (node instanceof SubsetNode) {
                addSubset(((SubsetNode) node).getSubsetKey());
            } else if (node instanceof GroupNode) {
                scanGroup((GroupNode) node);
            }
        }

    }

    private void scanGroup(GroupNode groupNode) {
        for (SubsetTree.Node node : groupNode.getSubnodeList()) {
            if (node instanceof SubsetNode) {
                addSubset(((SubsetNode) node).getSubsetKey());
            } else if (node instanceof GroupNode) {
                scanGroup((GroupNode) node);
            }
        }
    }

    private void addSubset(SubsetKey subsetKey) {
        Subset subset = bdfServer.getFichotheque().getSubset(subsetKey);
        if (subset == null) {
            return;
        }
        if (subset instanceof Corpus) {
            Corpus corpus = (Corpus) subset;
            List<CorpusInfo> destinationList;
            Subset masterSubset = corpus.getMasterSubset();
            boolean isSatellite;
            if ((masterSubset != null) && (masterSubset instanceof Corpus)) {
                destinationList = satelliteCorpusList;
                isSatellite = true;
            } else {
                destinationList = corpusList;
                isSatellite = false;
            }
            UmlClass umlClass = BdfPumlUtils.buildCorpusUmlClass(corpus, lang, messageLocalisation);
            CorpusInfo corpusInfo = new CorpusInfo(corpus, umlClass, destinationList.size(), isSatellite);
            subsetInfoMap.put(subsetKey, corpusInfo);
            destinationList.add(corpusInfo);
        } else {
            List<SubsetInfo> destinationList;
            if (subset instanceof Thesaurus) {
                destinationList = thesaurusList;
            } else {
                destinationList = otherList;
            }
            UmlClass umlClass = BdfPumlUtils.buildUmlClass(subset, lang);
            SubsetInfo subsetInfo = new SubsetInfo(subset, umlClass, destinationList.size());
            subsetInfoMap.put(subsetKey, subsetInfo);
            destinationList.add(subsetInfo);
        }
    }


    public static String toString(BdfServer bdfServer, Lang lang, MessageLocalisation messageLocalisation) {
        FichothequePumlWriter fichothequePumlWriter = new FichothequePumlWriter(bdfServer, lang, messageLocalisation);
        try {
            StringBuilder buf = new StringBuilder();
            fichothequePumlWriter.write(buf);
            return buf.toString();
        } catch (IOException ioe) {
            throw new ShouldNotOccurException();
        }
    }


    private static class SubsetInfo {

        private final Subset subset;
        private final UmlClass umlClass;
        protected final int index;
        private final Set<IncludeKey> relationSet = new LinkedHashSet<IncludeKey>();

        protected SubsetInfo(Subset subset, UmlClass umlClass, int index) {
            this.subset = subset;
            this.umlClass = umlClass;
            this.index = index;
        }

        public SubsetKey getSubsetKey() {
            return subset.getSubsetKey();
        }


        public boolean isEmpty() {
            return relationSet.isEmpty();
        }

        public void addIncludeKey(IncludeKey includeKey) {
            relationSet.add(includeKey);
        }

        public Set<IncludeKey> getRelationSet() {
            return relationSet;
        }

    }


    private static class CorpusInfo extends SubsetInfo {

        private final Corpus corpus;
        private final boolean isSatellite;
        private boolean withThesaurusRelation;

        protected CorpusInfo(Corpus corpus, UmlClass umlClass, int index, boolean isSatellite) {
            super(corpus, umlClass, index);
            this.corpus = corpus;
            this.isSatellite = isSatellite;
        }

        public Corpus getCorpus() {
            return corpus;
        }

        public void setWithThesaurusRelation(boolean withThesaurusRelation) {
            this.withThesaurusRelation = withThesaurusRelation;
        }

        public boolean isWithThesaurusRelation() {
            return withThesaurusRelation;
        }

        public boolean isBefore(CorpusInfo corpusSubsetInfo) {
            if (isSatellite) {
                if (corpusSubsetInfo.isSatellite) {
                    return (index <= corpusSubsetInfo.index);
                } else {
                    return false;
                }
            } else {
                if (corpusSubsetInfo.isSatellite) {
                    return true;
                } else {
                    return (index <= corpusSubsetInfo.index);
                }
            }
        }

    }

}
