/* BdfServer_Html - Copyright (c) 2017-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.html.jslib;

import fr.exemole.bdfserver.tools.storage.StorageUtils;
import java.text.ParseException;
import java.util.AbstractList;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.RandomAccess;
import net.mapeadores.util.text.RelativePath;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class JsLibBuilder {

    private final static List<JsLib> EMPTY_DEPENDCYLIST = Collections.emptyList();
    private final static List<ThirdLib> EMPTY_THIRDLIBLIST = Collections.emptyList();
    private final static List<JsrenderTemplateFamily> EMPTY_JSRENDERTEMPLATEFAMILYLIST = Collections.emptyList();
    private final static JsrenderTemplateFamily BDF_TEMPLATEFAMILY = new JsrenderTemplateFamily("bdf");
    private final List<JsLib> dependencyList = new ArrayList<JsLib>();
    private final List<RelativePath> jsScriptList = new ArrayList<RelativePath>();
    private final List<ThirdLib> list = new ArrayList<ThirdLib>();
    private final List<JsrenderTemplateFamily> templateList = new ArrayList<JsrenderTemplateFamily>();
    private String currentExtensionName = null;
    private String currentAppName = null;


    public JsLibBuilder() {

    }

    public JsLibBuilder setCurrentExtensionName(String currentExtensionName) {
        this.currentExtensionName = currentExtensionName;
        return this;
    }

    public JsLibBuilder setCurrentAppName(String currentAppName) {
        this.currentAppName = currentAppName;
        return this;
    }

    public JsLibBuilder addDependency(JsLib dependency) {
        dependencyList.add(dependency);
        return this;
    }

    public JsLibBuilder addThirdLib(String name, String... extensionNames) {
        List<String> extensionNameList;
        if ((extensionNames == null) || (extensionNames.length == 0)) {
            extensionNameList = StringUtils.EMPTY_STRINGLIST;
        } else {
            extensionNameList = StringUtils.wrap(extensionNames);
        }
        list.add(new ThirdLib(name, extensionNameList));
        return this;
    }

    public JsLibBuilder addJs(String jsPath) {
        try {
            RelativePath relativePath = RelativePath.parse("js/" + jsPath);
            jsScriptList.add(relativePath);
        } catch (ParseException pe) {

        }
        return this;
    }

    public JsLibBuilder addExtensionJs(String jsPath) {
        if (currentExtensionName == null) {
            throw new IllegalStateException("currentExtensionName is null");
        }
        try {
            RelativePath relativePath = StorageUtils.parseExtensionResourcePath(currentExtensionName, "js/" + jsPath);
            jsScriptList.add(relativePath);
        } catch (ParseException pe) {

        }
        return this;
    }

    public JsLibBuilder addAppJs(String jsPath) {
        if (currentAppName == null) {
            throw new IllegalStateException("currentExtensionName is null");
        }
        try {
            RelativePath relativePath = RelativePath.parse("apps/" + currentAppName + "/js/" + jsPath);
            jsScriptList.add(relativePath);
        } catch (ParseException pe) {

        }
        return this;
    }


    public JsLibBuilder addTemplateFamily(String family) {
        templateList.add(new JsrenderTemplateFamily(family));
        return this;
    }

    public JsLibBuilder addExtensionTemplateFamily(String family) {
        if (currentExtensionName == null) {
            throw new IllegalStateException("currentExtensionName is null");
        }
        templateList.add(new JsrenderTemplateFamily(family, JsrenderTemplateFamily.EXTENSION_TYPE, currentExtensionName));
        return this;
    }

    public JsLibBuilder addAppTemplateFamily(String family) {
        if (currentAppName == null) {
            throw new IllegalStateException("currentAppName is null");
        }
        templateList.add(new JsrenderTemplateFamily(family, JsrenderTemplateFamily.APP_TYPE, currentAppName));
        return this;
    }

    public List<ThirdLib> toThirdLibList() {
        return new ThirdLibList(list.toArray(new ThirdLib[list.size()]));
    }

    public JsLib toJsLib() {
        List<JsLib> finalDependencyList;
        if (dependencyList.isEmpty()) {
            finalDependencyList = EMPTY_DEPENDCYLIST;
        } else {
            finalDependencyList = new JsLibList(dependencyList.toArray(new JsLib[dependencyList.size()]));
        }
        List<RelativePath> finalJsScriptList;
        if (jsScriptList.isEmpty()) {
            finalJsScriptList = StringUtils.EMPTY_RELATIVEPATHLIST;
        } else {
            finalJsScriptList = StringUtils.wrap(jsScriptList.toArray(new RelativePath[jsScriptList.size()]));
        }
        List<ThirdLib> finalThirdLibList;
        if (list.isEmpty()) {
            finalThirdLibList = EMPTY_THIRDLIBLIST;
        } else {
            finalThirdLibList = new ThirdLibList(list.toArray(new ThirdLib[list.size()]));
        }
        List<JsrenderTemplateFamily> finalJsrenderTemplateList;
        if (templateList.isEmpty()) {
            finalJsrenderTemplateList = EMPTY_JSRENDERTEMPLATEFAMILYLIST;
        } else {
            int size = templateList.size();
            JsrenderTemplateFamily[] array = new JsrenderTemplateFamily[size + 1];
            array[0] = BDF_TEMPLATEFAMILY;
            int p = 1;
            for (JsrenderTemplateFamily family : templateList) {
                array[p] = family;
                p++;
            }
            finalJsrenderTemplateList = new JsrenderTemplateList(array);
        }
        return new InternalJsLib(finalDependencyList, finalJsScriptList, finalThirdLibList, finalJsrenderTemplateList);
    }

    public static JsLibBuilder init() {
        return new JsLibBuilder();
    }


    private static class InternalJsLib extends JsLib {

        private final List<JsLib> dependencyList;
        private final List<RelativePath> jsScriptList;
        private final List<ThirdLib> thirdLibList;
        private final List<JsrenderTemplateFamily> templateList;

        private InternalJsLib(List<JsLib> dependencyList, List<RelativePath> jsScriptList, List<ThirdLib> thirdLibList, List<JsrenderTemplateFamily> templateList) {
            this.dependencyList = dependencyList;
            this.jsScriptList = jsScriptList;
            this.thirdLibList = thirdLibList;
            this.templateList = templateList;
        }

        @Override
        public List<JsLib> getDependencyList() {
            return dependencyList;
        }

        @Override
        public List<RelativePath> getJsScriptList() {
            return jsScriptList;
        }

        @Override
        public List<ThirdLib> getThirdLibList() {
            return thirdLibList;
        }

        @Override
        public List<JsrenderTemplateFamily> getJsrenderTemplateFamilyList() {
            return templateList;
        }

    }


    private static class JsLibList extends AbstractList<JsLib> implements RandomAccess {

        private final JsLib[] array;

        private JsLibList(JsLib[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public JsLib get(int index) {
            return array[index];
        }

    }


    private static class ThirdLibList extends AbstractList<ThirdLib> implements RandomAccess {

        private final ThirdLib[] array;

        private ThirdLibList(ThirdLib[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public ThirdLib get(int index) {
            return array[index];
        }

    }


    private static class JsrenderTemplateList extends AbstractList<JsrenderTemplateFamily> implements RandomAccess {

        private final JsrenderTemplateFamily[] array;

        private JsrenderTemplateList(JsrenderTemplateFamily[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public JsrenderTemplateFamily get(int index) {
            return array[index];
        }

    }


}
