/* BdfServer_HtmlProducers - Copyright (c) 2010-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.edition;

import fr.exemole.bdfserver.api.ficheform.FicheForm;
import fr.exemole.bdfserver.api.ficheform.FormElement;
import fr.exemole.bdfserver.api.instruction.ProducerParameters;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.interaction.InteractionConstants;
import fr.exemole.bdfserver.api.interaction.domains.EditionDomain;
import fr.exemole.bdfserver.api.namespaces.BdfUserSpace;
import fr.exemole.bdfserver.api.ui.CommentUi;
import fr.exemole.bdfserver.api.ui.FieldUi;
import fr.exemole.bdfserver.api.ui.IncludeUi;
import fr.exemole.bdfserver.api.ui.UiComponent;
import fr.exemole.bdfserver.api.ui.UiComponents;
import fr.exemole.bdfserver.commands.edition.FicheChangeCommand;
import fr.exemole.bdfserver.commands.edition.FicheCreationCommand;
import fr.exemole.bdfserver.html.BdfServerHtmlProducer;
import fr.exemole.bdfserver.html.ParameterMap;
import fr.exemole.bdfserver.html.consumers.Button;
import fr.exemole.bdfserver.html.forms.CommonFormHtml;
import fr.exemole.bdfserver.html.forms.CorpusFieldFormHtml;
import fr.exemole.bdfserver.html.jslib.FicheJsLibs;
import fr.exemole.bdfserver.tools.users.BdfUserUtils;
import java.util.ArrayList;
import java.util.List;
import net.fichotheque.SubsetItem;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.utils.CorpusMetadataUtils;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.JsObject;
import net.mapeadores.util.html.TrustedHtml;


/**
 *
 * @author Vincent Calame
 */
public class FicheFormHtmlProducer extends BdfServerHtmlProducer {

    private final Corpus corpus;
    private String title;
    private SubsetItem masterSubsetItem = null;
    private final FicheForm ficheForm;
    private List<Object> elementList;
    private String ficheResultPage = EditionDomain.FICHE_RESULT_PAGE;
    private String commandName;
    private boolean discarded = false;
    int id = -1;

    private FicheFormHtmlProducer(ProducerParameters producerParameters, Corpus corpus, FicheForm ficheForm) {
        super(producerParameters);
        this.corpus = corpus;
        this.ficheForm = ficheForm;
        addJsLib(FicheJsLibs.FORM);
        addThemeCss("ficheform.css", "edition.css");
        setMainStorageKey("FF_" + corpus.getSubsetKey().toString());
    }

    public void setFicheResultPage(String ficheResultPage) {
        this.ficheResultPage = ficheResultPage;
    }

    @Override
    public void printHtml() {
        String formId = generateId();
        ParameterMap parameterMap = ParameterMap.init()
                .command(commandName)
                .page(ficheResultPage)
                .subset(corpus);
        if (id != -1) {
            parameterMap.param(InteractionConstants.ID_PARAMNAME, String.valueOf(id));
        }
        start();
        this
                .SCRIPT()
                .__jsObject("Ficheform.ARGS", initFicheformArgs())
                ._SCRIPT();
        this
                .DIV("edition-Page edition-form-Page")
                .__(EditionHtmlUtils.printFicheHeader(this, title, discarded, formId))
                .MAIN("edition-Unit")
                .FORM_post(HA.action(Domains.EDITION).id(formId)
                        .attr("data-ficheform-role", "form")
                        .attr("data-ficheform-corpus", corpus.getSubsetKey().getSubsetName()))
                .INPUT_hidden(parameterMap)
                .DIV("edition-form-Entries")
                .__(printFormEntries())
                .__(Button.COMMAND,
                        Button.submit("action-Save", "_ link.global.ok").formId(formId).ficheForm(true))
                ._DIV()
                ._FORM()
                ._MAIN()
                ._DIV();
        end();
    }

    private boolean printFormEntries() {
        for (Object obj : elementList) {
            if (obj instanceof FormElement) {
                this
                        .__(CommonFormHtml.printFormElement(bdfServer, this, (FormElement) obj, null, bdfUser));
            } else if (obj instanceof CommentUi) {
                this
                        .__(printCommentUi((CommentUi) obj));
            }
        }
        return true;
    }

    private void initElementList(UiComponents uiComponents) {
        elementList = new ArrayList();
        for (UiComponent uiComponent : uiComponents.getUiComponentList()) {
            if (uiComponent instanceof FieldUi) {
                FormElement.Field formElement = ficheForm.getFormElement((FieldUi) uiComponent);
                if (formElement != null) {
                    elementList.add(formElement);
                }
            } else if (uiComponent instanceof IncludeUi) {
                FormElement.Include includeElement = ficheForm.getFormElement((IncludeUi) uiComponent);
                if (includeElement != null) {
                    elementList.add(includeElement);
                }
            } else if (uiComponent instanceof CommentUi) {
                elementList.add((CommentUi) uiComponent);
            } else {
                throw new IllegalArgumentException("unknown ComponentUi implementation : " + uiComponent.getClass().getName());
            }
        }
    }

    private boolean printCommentUi(CommentUi commentUi) {
        if ((commentUi.getLocation() & CommentUi.FORM_BITVALUE) != 0) {
            TrustedHtml html = commentUi.getHtmlByLang(workingLang);
            if (html != null) {
                CorpusFieldFormHtml.printComment(this, html);
            }
        }
        return true;
    }

    private JsObject initFicheformArgs() {
        boolean noTab = BdfUserUtils.getBoolean(bdfUser, BdfUserSpace.NOTAB_KEY);
        if (!noTab) {
            Attribute attribute = bdfUser.getRedacteur().getAttributes().getAttribute(BdfUserSpace.FICHEFORM_KEY);
            if (attribute != null) {
                for (String value : attribute) {
                    if (value.equals("noTab")) {
                        noTab = true;
                    }
                }
            }
        }
        return JsObject.init()
                .put("noTab", noTab);
    }

    public static FicheFormHtmlProducer newCreationInstance(ProducerParameters producerParameters, Corpus corpus, FicheForm ficheForm, UiComponents uiComponents) {
        FicheFormHtmlProducer ficheFormHtmlProducer = new FicheFormHtmlProducer(producerParameters, corpus, ficheForm);
        ficheFormHtmlProducer.commandName = FicheCreationCommand.COMMANDNAME;
        ficheFormHtmlProducer.title = CorpusMetadataUtils.getNewFicheLabel(corpus, ficheFormHtmlProducer.workingLang);
        ficheFormHtmlProducer.initElementList(uiComponents);
        return ficheFormHtmlProducer;
    }

    public static FicheFormHtmlProducer newCreationInstance(ProducerParameters producerParameters, Corpus corpus, int id, FicheForm ficheForm, UiComponents uiComponents) {
        FicheFormHtmlProducer ficheFormHtmlProducer = new FicheFormHtmlProducer(producerParameters, corpus, ficheForm);
        ficheFormHtmlProducer.commandName = FicheChangeCommand.COMMANDNAME;
        ficheFormHtmlProducer.id = id;
        ficheFormHtmlProducer.title = CorpusMetadataUtils.getNewFicheLabel(corpus, ficheFormHtmlProducer.workingLang) + " (id=" + String.valueOf(id) + ")";
        ficheFormHtmlProducer.initElementList(uiComponents);
        return ficheFormHtmlProducer;
    }

    public static FicheFormHtmlProducer newCreationInstance(ProducerParameters producerParameters, Corpus corpus, SubsetItem masterSubsetItem, FicheForm ficheForm, UiComponents uiComponents) {
        FicheFormHtmlProducer ficheFormHtmlProducer = new FicheFormHtmlProducer(producerParameters, corpus, ficheForm);
        ficheFormHtmlProducer.commandName = FicheChangeCommand.COMMANDNAME;
        ficheFormHtmlProducer.id = masterSubsetItem.getId();
        ficheFormHtmlProducer.masterSubsetItem = masterSubsetItem;
        ficheFormHtmlProducer.title = CorpusMetadataUtils.getNewFicheLabel(corpus, ficheFormHtmlProducer.workingLang) + " (id=" + String.valueOf(masterSubsetItem.getId()) + ")";
        ficheFormHtmlProducer.initElementList(uiComponents);
        return ficheFormHtmlProducer;
    }

    public static FicheFormHtmlProducer newChangeInstance(ProducerParameters producerParameters, FicheMeta ficheMeta, FicheForm ficheForm, UiComponents uiComponents) {
        FicheFormHtmlProducer ficheFormHtmlProducer = new FicheFormHtmlProducer(producerParameters, ficheMeta.getCorpus(), ficheForm);
        ficheFormHtmlProducer.commandName = FicheChangeCommand.COMMANDNAME;
        ficheFormHtmlProducer.id = ficheMeta.getId();
        ficheFormHtmlProducer.title = CorpusMetadataUtils.getFicheTitle(ficheMeta, ficheFormHtmlProducer.workingLang, ficheFormHtmlProducer.formatLocale);
        ficheFormHtmlProducer.discarded = ficheMeta.isDiscarded();
        ficheFormHtmlProducer.initElementList(uiComponents);
        return ficheFormHtmlProducer;
    }

}
