/* BdfServer_JsonProducers - Copyright (c) 2012-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.jsonproducers.exportation;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.configuration.PathConfiguration;
import fr.exemole.bdfserver.api.instruction.BdfCommandResult;
import fr.exemole.bdfserver.api.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.instruction.OutputParameters;
import fr.exemole.bdfserver.api.interaction.domains.ExportationDomain;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.jsonproducers.ResultJsonProducer;
import fr.exemole.bdfserver.tools.balayage.engine.BalayageLog;
import fr.exemole.bdfserver.tools.configuration.PathConfigurationBuilder;
import fr.exemole.bdfserver.tools.instruction.BdfCommandResultBuilder;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import java.text.ParseException;
import java.util.LinkedHashSet;
import java.util.Set;
import net.fichotheque.exportation.balayage.BalayageContentDescription;
import net.fichotheque.exportation.balayage.BalayageDescription;
import net.fichotheque.exportation.scrutari.ScrutariExportDef;
import net.fichotheque.exportation.sql.SqlExportDef;
import net.fichotheque.exportation.table.TableExportContentDescription;
import net.fichotheque.exportation.table.TableExportDescription;
import net.fichotheque.exportation.transformation.TemplateContentDescription;
import net.fichotheque.exportation.transformation.TemplateDescription;
import net.fichotheque.exportation.transformation.TransformationDescription;
import net.fichotheque.exportation.transformation.TransformationKey;
import net.fichotheque.tools.exportation.scrutari.ScrutariExportUtils;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.json.JsonProperty;
import net.mapeadores.util.json.JsonUtils;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.MessageLocalisation;
import net.mapeadores.util.logging.MessageByLineLog;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public final class ExportationJsonProducerFactory {

    private ExportationJsonProducerFactory() {
    }

    public static JsonProducer getJsonProducer(OutputParameters parameters) throws BdfInstructionException {
        String json = parameters.getOutput();
        BdfCommandResult bdfCommandResult = parameters.getBdfCommandResult();
        BdfServer bdfServer = parameters.getBdfServer();
        BdfUser bdfUser = parameters.getBdfUser();
        RequestMap requestMap = parameters.getRequestMap();
        Lang lang = bdfUser.getWorkingLang();
        MessageLocalisation messageLocalisation = bdfServer.getL10nManager().getMessageLocalisation(bdfUser.getLangPreference());
        JsonProperty jsonProperty = null;
        switch (json) {
            case ExportationDomain.TABLEEXPORT_JSON:
                jsonProperty = new TableExportJsonProperty(getTableExportDescription(bdfServer, requestMap, bdfCommandResult), lang, messageLocalisation);
                break;
            case ExportationDomain.TABLEEXPORT_ARRAY_JSON:
                bdfServer.getTableExportManager().update();
                jsonProperty = new TableExportArrayJsonProperty(bdfServer, lang, messageLocalisation);
                break;
            case ExportationDomain.TABLEEXPORT_CONTENT_JSON:
                jsonProperty = new TableExportContentJsonProperty(bdfServer, getTableExportContentDescription(bdfServer, requestMap), messageLocalisation);
                break;
            case ExportationDomain.SCRUTARIEXPORT_JSON:
                ScrutariExportDef scrutariExportDef = getScrutariExportDef(bdfServer, requestMap, bdfCommandResult);
                jsonProperty = new ScrutariExportJsonProperty(scrutariExportDef, lang);
                if (bdfCommandResult == null) {
                    MessageByLineLog messageByLineLog = ScrutariExportUtils.checkFieldGeneration(scrutariExportDef, bdfServer.getTableExportContext());
                    if (!messageByLineLog.getLogGroupList().isEmpty()) {
                        bdfCommandResult = BdfCommandResultBuilder.init()
                                .putResultObject(BdfInstructionConstants.MESSAGEBYLINELOG_OBJ, messageByLineLog)
                                .toBdfCommandResult();
                    }
                }
                break;
            case ExportationDomain.SCRUTARIEXPORT_ARRAY_JSON:
                jsonProperty = new ScrutariExportArrayJsonProperty(bdfServer, lang);
                break;
            case ExportationDomain.SCRUTARIEXPORT_PATHS_JSON:
                jsonProperty = new ScrutariExportPathsJsonProperty(getScrutariExportDef(bdfServer, requestMap, bdfCommandResult), getPathConfiguration(bdfServer, bdfCommandResult));
                break;
            case ExportationDomain.TRANSFORMATION_DESCRIPTION_JSON:
                jsonProperty = new TransformationDescriptionJsonProperty(getTransformationDescription(bdfServer, requestMap, bdfCommandResult), lang, messageLocalisation);
                break;
            case ExportationDomain.TRANSFORMATION_DESCRIPTIONMAP_JSON:
                bdfServer.getTransformationManager().update();
                jsonProperty = new TransformationDescriptionMapJsonProperty(bdfServer.getTransformationManager(), getTransformationKeySet(bdfServer, requestMap), lang, messageLocalisation);
                break;
            case ExportationDomain.TRANSFORMATION_TEMPLATEDESCRIPTION_JSON:
                jsonProperty = new TemplateDescriptionJsonProperty(getTemplateDescription(bdfServer, requestMap, bdfCommandResult), lang, messageLocalisation);
                break;
            case ExportationDomain.TRANSFORMATION_TEMPLATECONTENT_JSON:
                jsonProperty = new TemplateContentJsonProperty(bdfServer, getTemplateContentDescription(bdfServer, requestMap), messageLocalisation);
                break;
            case ExportationDomain.SQLEXPORT_JSON:
                jsonProperty = new SqlExportJsonProperty(getSqlExportDef(bdfServer, requestMap, bdfCommandResult), lang);
                break;
            case ExportationDomain.SQLEXPORT_ARRAY_JSON:
                jsonProperty = new SqlExportArrayJsonProperty(bdfServer, lang);
                break;
            case ExportationDomain.SQLEXPORT_PATHS_JSON:
                jsonProperty = new SqlExportPathsJsonProperty(getSqlExportDef(bdfServer, requestMap, bdfCommandResult), getPathConfiguration(bdfServer, bdfCommandResult));
                break;
            case ExportationDomain.BALAYAGE_JSON:
                jsonProperty = new BalayageJsonProperty(getBalayageDescription(bdfServer, requestMap, bdfCommandResult), lang, messageLocalisation);
                break;
            case ExportationDomain.BALAYAGE_ARRAY_JSON:
                bdfServer.getBalayageManager().update();
                jsonProperty = new BalayageArrayJsonProperty(bdfServer, lang, messageLocalisation);
                break;
            case ExportationDomain.BALAYAGE_CONTENT_JSON:
                jsonProperty = new BalayageContentJsonProperty(bdfServer, getBalayageContentDescription(bdfServer, requestMap), messageLocalisation);
                break;
            case ExportationDomain.BALAYAGE_LOG_JSON:
                BalayageLog balayageLog = getBalayageLog(bdfCommandResult);
                if (balayageLog != null) {
                    jsonProperty = balayageLog;
                } else {
                    jsonProperty = JsonUtils.EMPTY_JSONPROPERTY;
                }
                break;
        }
        if (jsonProperty != null) {
            ResultJsonProducer jsonProducer = new ResultJsonProducer(bdfCommandResult, messageLocalisation);
            jsonProducer.add(jsonProperty);
            return jsonProducer;
        } else {
            return null;
        }
    }

    private static BalayageDescription getBalayageDescription(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws BdfInstructionException {
        BalayageDescription balayageDescription = null;
        if (bdfCommandResult != null) {
            balayageDescription = (BalayageDescription) bdfCommandResult.getResultObject(BdfInstructionConstants.BALAYAGEDESCRIPTION_OBJ);
        }
        if (balayageDescription == null) {
            balayageDescription = BdfInstructionUtils.getMandatoryBalayageDescription(bdfServer.getBalayageManager(), requestMap);
        }
        return balayageDescription;
    }

    private static BalayageContentDescription getBalayageContentDescription(BdfServer bdfServer, RequestMap requestMap) throws BdfInstructionException {
        BalayageDescription balayageDescription = BdfInstructionUtils.getMandatoryBalayageDescription(bdfServer.getBalayageManager(), requestMap);
        return BdfInstructionUtils.getMandatoryBalayageContentDescription(balayageDescription, requestMap);
    }

    private static BalayageLog getBalayageLog(BdfCommandResult bdfCommandResult) {
        if (bdfCommandResult != null) {
            return (BalayageLog) bdfCommandResult.getResultObject(BdfInstructionConstants.BALAYAGELOG_OBJ);
        }
        return null;
    }

    private static ScrutariExportDef getScrutariExportDef(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws BdfInstructionException {
        ScrutariExportDef scrutariExportDef = null;
        if (bdfCommandResult != null) {
            scrutariExportDef = (ScrutariExportDef) bdfCommandResult.getResultObject(BdfInstructionConstants.SCRUTARIEXPORTDEF_OBJ);
        }
        if (scrutariExportDef == null) {
            scrutariExportDef = BdfInstructionUtils.getMandatoryScrutariExportDef(bdfServer.getScrutariExportManager(), requestMap);
        }
        return scrutariExportDef;
    }

    private static SqlExportDef getSqlExportDef(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws BdfInstructionException {
        SqlExportDef sqlExportDef = null;
        if (bdfCommandResult != null) {
            sqlExportDef = (SqlExportDef) bdfCommandResult.getResultObject(BdfInstructionConstants.SQLEXPORTDEF_OBJ);
        }
        if (sqlExportDef == null) {
            sqlExportDef = BdfInstructionUtils.getMandatorySqlExportDef(bdfServer.getSqlExportManager(), requestMap);
        }
        return sqlExportDef;
    }

    private static TableExportDescription getTableExportDescription(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws BdfInstructionException {
        TableExportDescription tableExportDescription = null;
        if (bdfCommandResult != null) {
            tableExportDescription = (TableExportDescription) bdfCommandResult.getResultObject(BdfInstructionConstants.TABLEEXPORTDESCRIPTION_OBJ);
        }
        if (tableExportDescription == null) {
            tableExportDescription = BdfInstructionUtils.getMandatoryTableExportDescription(bdfServer.getTableExportManager(), requestMap);
        }
        return tableExportDescription;
    }

    private static TableExportContentDescription getTableExportContentDescription(BdfServer bdfServer, RequestMap requestMap) throws BdfInstructionException {
        TableExportDescription tableExportDescription = BdfInstructionUtils.getMandatoryTableExportDescription(bdfServer.getTableExportManager(), requestMap);
        return BdfInstructionUtils.getMandatoryTableExportContentDescription(tableExportDescription, requestMap);
    }

    private static TransformationDescription getTransformationDescription(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws BdfInstructionException {
        TransformationDescription transformationDescription = null;
        if (bdfCommandResult != null) {
            transformationDescription = (TransformationDescription) bdfCommandResult.getResultObject(BdfInstructionConstants.TRANSFORMATIONDESCRIPTION_OBJ);
        }
        if (transformationDescription == null) {
            transformationDescription = BdfInstructionUtils.getMandatoryTransformationDescription(bdfServer, requestMap);
        }
        return transformationDescription;
    }

    private static PathConfiguration getPathConfiguration(BdfServer bdfServer, BdfCommandResult bdfCommandResult) {
        if (bdfCommandResult != null) {
            PathConfiguration pathConfiguration = (PathConfiguration) bdfCommandResult.getResultObject(BdfInstructionConstants.PATHCONFIGURATION_OBJ);
            if (pathConfiguration != null) {
                return pathConfiguration;
            }
        }
        return PathConfigurationBuilder.build(bdfServer);
    }

    private static Set<TransformationKey> getTransformationKeySet(BdfServer bdfServer, RequestMap requestMap) throws BdfInstructionException {
        String[] keysArray = BdfInstructionUtils.getMandatoryParameterValues(requestMap, "keys");
        Set<TransformationKey> transformationKeySet = new LinkedHashSet<TransformationKey>();
        for (String keys : keysArray) {
            String[] tokens = StringUtils.getTechnicalTokens(keys, true);
            for (String token : tokens) {
                try {
                    transformationKeySet.add(TransformationKey.parse(token));
                } catch (ParseException pe) {
                    throw new BdfInstructionException(BdfInstructionUtils.unknownParameterValue("keys", token));
                }
            }
        }
        return transformationKeySet;
    }

    private static TemplateDescription getTemplateDescription(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws BdfInstructionException {
        TemplateDescription templateDescription = null;
        if (bdfCommandResult != null) {
            templateDescription = (TemplateDescription) bdfCommandResult.getResultObject(BdfInstructionConstants.TEMPLATEDESCRIPTION_OBJ);
        }
        if (templateDescription == null) {
            templateDescription = BdfInstructionUtils.getMandatoryTemplateDescription(bdfServer, requestMap);
        }
        return templateDescription;
    }

    private static TemplateContentDescription getTemplateContentDescription(BdfServer bdfServer, RequestMap requestMap) throws BdfInstructionException {
        TemplateDescription templateDescription = BdfInstructionUtils.getMandatoryTemplateDescription(bdfServer, requestMap);
        return BdfInstructionUtils.getMandatoryTemplateContentDescription(templateDescription, requestMap);
    }

}
