/* FichothequeLib_API - Copyright (c) 2017-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.utils.selection;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.function.Predicate;
import net.fichotheque.Fichotheque;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.croisement.Croisements;
import net.fichotheque.selection.FicheCondition;
import net.fichotheque.selection.FicheSelector;
import net.fichotheque.selection.SelectionContext;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.utils.SelectionUtils;
import net.mapeadores.util.conditions.ConditionsConstants;
import net.fichotheque.selection.FicheQuery;


/**
 *
 * @author Vincent Calame
 */
public final class FicheConditionPredicateFactory {

    private final static Test ALWAYS_TEST = new ConstantTest(true);

    private FicheConditionPredicateFactory() {
    }

    public static Predicate<Motcle> newInstance(SelectionContext selectionContext, FicheCondition ficheCondition) {
        Test[] testArray;
        List<Test> testList = new ArrayList<Test>();
        for (FicheQuery ficheQuery : ficheCondition.getFicheQueryList()) {
            Test test = toTest(selectionContext, ficheQuery);
            if (test != null) {
                testList.add(test);
            }
        }
        int size = testList.size();
        if (size == 0) {
            testArray = new Test[1];
            testArray[0] = ALWAYS_TEST;
        } else {
            testArray = testList.toArray(new Test[size]);
        }
        return new ResultPredicate(ficheCondition.getSelectMode(), testArray);
    }

    private static Test toTest(SelectionContext selectionContext, FicheQuery ficheQuery) {
        boolean invert = false;
        if (SelectionUtils.isEmpty(ficheQuery)) {
            return null;
        } else if (SelectionUtils.isOnlyList(ficheQuery)) {
            return new CorpusListTest(invert, selectionContext, ficheQuery);
        } else {
            return new FicheSelectorTest(invert, selectionContext, ficheQuery);
        }
    }

    private static Set<SubsetKey> toSubsetKeySet(Corpus[] corpusArray) {
        Set<SubsetKey> subsetKeySet = new HashSet<SubsetKey>();
        for (Corpus corpus : corpusArray) {
            subsetKeySet.add(corpus.getSubsetKey());
        }
        return subsetKeySet;
    }


    private static class ResultPredicate implements Predicate<Motcle> {

        private final short optionsMode;
        private final Test[] testArray;

        private ResultPredicate(short mode, Test[] testArray) {
            this.optionsMode = mode;
            this.testArray = testArray;
        }

        @Override
        public boolean test(Motcle motcle) {
            if (optionsMode == ConditionsConstants.MODE_OR) {
                for (Test test : testArray) {
                    if (test.isSelected(motcle)) {
                        return true;
                    }
                }
                return false;
            } else {
                for (Test test : testArray) {
                    if (!test.isSelected(motcle)) {
                        return false;
                    }
                }
                return true;
            }

        }

    }


    private static abstract class Test {

        private final boolean invert;

        protected Test(boolean invert) {
            this.invert = invert;
        }

        public boolean isSelected(Motcle motcle) {
            boolean test = test(motcle);
            if (invert) {
                return !test;
            } else {
                return test;
            }
        }

        protected abstract boolean test(Motcle motcle);

    }


    private static class ConstantTest extends Test {

        private ConstantTest(boolean constant) {
            super(constant);
        }

        @Override
        public boolean test(Motcle motcle) {
            return true;
        }

    }


    private static class CorpusListTest extends Test {

        private final Fichotheque fichotheque;
        private final Corpus[] corpusArray;
        private final boolean withSatelliteTest;
        private final Predicate<FicheMeta> fichePredicate;
        private final Set<SubsetKey> subsetKeySet;

        private CorpusListTest(boolean invert, SelectionContext selectionContext, FicheQuery ficheQuery) {
            super(invert);
            this.fichotheque = selectionContext.getFichotheque();
            this.corpusArray = SelectionUtils.toCorpusArray(fichotheque, ficheQuery, selectionContext.getSubsetAccessPredicate());
            this.withSatelliteTest = ficheQuery.isWithRattachement();
            this.fichePredicate = selectionContext.getFichePredicate();
            if (withSatelliteTest) {
                this.subsetKeySet = toSubsetKeySet(corpusArray);
            } else {
                this.subsetKeySet = null;
            }
        }

        @Override
        public boolean test(Motcle motcle) {
            if (withSatelliteTest) {
                for (Corpus corpus : motcle.getThesaurus().getSatelliteCorpusList()) {
                    if (subsetKeySet.contains(corpus.getSubsetKey())) {
                        FicheMeta ficheMeta = corpus.getFicheMetaById(motcle.getId());
                        if (ficheMeta != null) {
                            if (fichePredicate.test(ficheMeta)) {
                                return true;
                            }
                        }
                    }
                }
            }
            for (Corpus corpus : corpusArray) {
                Croisements croisements = fichotheque.getCroisements(motcle, corpus);
                if (!croisements.isEmpty()) {
                    for (Croisements.Entry entry : croisements.getEntryList()) {
                        FicheMeta ficheMeta = (FicheMeta) entry.getSubsetItem();
                        if (fichePredicate.test(ficheMeta)) {
                            return true;
                        }
                    }
                }
            }
            return false;
        }

    }


    private static class FicheSelectorTest extends Test {

        private final Fichotheque fichotheque;
        private final Corpus[] corpusArray;
        private final boolean withSatelliteTest;
        private final Set<SubsetKey> subsetKeySet;
        private final FicheSelector ficheSelector;
        private final boolean withCroisementCondition;

        private FicheSelectorTest(boolean invert, SelectionContext selectionContext, FicheQuery ficheQuery) {
            super(invert);
            this.fichotheque = selectionContext.getFichotheque();
            this.corpusArray = SelectionUtils.toCorpusArray(fichotheque, ficheQuery, selectionContext.getSubsetAccessPredicate());
            this.withSatelliteTest = ficheQuery.isWithRattachement();
            if (withSatelliteTest) {
                this.subsetKeySet = toSubsetKeySet(corpusArray);
            } else {
                this.subsetKeySet = null;
            }
            this.ficheSelector = FicheSelectorBuilder.init(selectionContext).add(ficheQuery).toFicheSelector();
            this.withCroisementCondition = (ficheQuery.getCroisementCondition() != null);
        }

        @Override
        public boolean test(Motcle motcle) {
            if (withSatelliteTest) {
                for (Corpus corpus : motcle.getThesaurus().getSatelliteCorpusList()) {
                    if (subsetKeySet.contains(corpus.getSubsetKey())) {
                        FicheMeta satelliteFicheMeta = corpus.getFicheMetaById(motcle.getId());
                        if (satelliteFicheMeta != null) {
                            if (ficheSelector.test(satelliteFicheMeta)) {
                                return true;
                            }
                        }
                    }
                }
            }
            for (Corpus corpus : corpusArray) {
                Croisements croisements = fichotheque.getCroisements(motcle, corpus);
                for (Croisements.Entry entry : croisements.getEntryList()) {
                    FicheMeta ficheMeta = (FicheMeta) entry.getSubsetItem();
                    if (!withCroisementCondition) {
                        if (ficheSelector.test(ficheMeta)) {
                            return true;
                        }
                    } else {
                        if (ficheSelector.isSelected(ficheMeta, entry.getCroisement()) != null) {
                            return true;
                        }
                    }
                }
            }
            return false;
        }

    }

}
