/* FichothequeLib_Tools - Copyright (c) 2012-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.extraction;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.croisement.Croisement;
import net.fichotheque.extraction.def.ExtraitCorpusDef;
import net.fichotheque.extraction.def.FicheFilter;
import net.fichotheque.extraction.def.GroupClause;
import net.fichotheque.extraction.def.IntituleClause;
import net.fichotheque.extraction.def.PackClause;
import net.fichotheque.extraction.run.CorpusExtractor;
import net.fichotheque.extraction.run.ExtraitCorpusResult;
import net.fichotheque.extraction.run.FicheGroup;
import net.fichotheque.tools.extraction.builders.BuilderUtils;
import net.fichotheque.tools.extraction.corpus.GroupEngine;


/**
 *
 * @author Vincent Calame
 */
class CorpusExtractorImpl implements CorpusExtractor {

    private final ResultBuilder namedResultBuilder;
    private final Map<SubsetKey, ResultBuilder> corpusResultBuilderMap;
    private final ExtraitCorpusDef extraitCorpusDef;
    private boolean containsFiche = false;

    CorpusExtractorImpl(ExtraitCorpusDef extraitCorpusDef) {
        this.extraitCorpusDef = extraitCorpusDef;
        String extraitName = extraitCorpusDef.getExtraitName();
        if (extraitName != null) {
            namedResultBuilder = new ResultBuilder(null);
            corpusResultBuilderMap = null;
        } else {
            namedResultBuilder = null;
            corpusResultBuilderMap = new TreeMap<SubsetKey, ResultBuilder>();
        }
    }

    @Override
    public void add(FicheMeta ficheMeta, Croisement croisement) {
        containsFiche = true;
        if (namedResultBuilder != null) {
            namedResultBuilder.add(ficheMeta, extraitCorpusDef.getFicheFilter(), croisement);
        } else {
            Corpus corpus = ficheMeta.getCorpus();
            ResultBuilder resultBuilder = corpusResultBuilderMap.get(corpus.getSubsetKey());
            if (resultBuilder == null) {
                resultBuilder = new ResultBuilder(corpus);
                corpusResultBuilderMap.put(corpus.getSubsetKey(), resultBuilder);
            }
            resultBuilder.add(ficheMeta, extraitCorpusDef.getFicheFilter(), croisement);
        }
    }

    @Override
    public boolean containsFiche() {
        return containsFiche;
    }

    @Override
    public List<ExtraitCorpusResult> getExtraitCorpusResultList() {
        if (!containsFiche) {
            return BuilderUtils.EMPTY_EXTRAITCORPUSRESULTLIST;
        }
        if (namedResultBuilder != null) {
            return Collections.singletonList(namedResultBuilder.getExtraitCorpusResult());
        } else {
            List<ExtraitCorpusResult> list = new ArrayList<ExtraitCorpusResult>();
            for (ResultBuilder resultBuilder : corpusResultBuilderMap.values()) {
                list.add(resultBuilder.getExtraitCorpusResult());
            }
            return list;
        }
    }


    private class ResultBuilder {

        private final Corpus corpus;
        private final GroupEngine groupEngine;

        private ResultBuilder(Corpus corpus) {
            this.corpus = corpus;
            this.groupEngine = GroupEngine.newInstance(extraitCorpusDef.getGroupClause());
        }

        private ExtraitCorpusResult getExtraitCorpusResult() {
            SubsetKey corpusKey = null;
            if (corpus != null) {
                corpusKey = corpus.getSubsetKey();
            }
            InternalExtraitCorpusResult internalExtraitCorpusResult = new InternalExtraitCorpusResult(extraitCorpusDef, corpusKey, groupEngine.toFicheGroupArray());
            return internalExtraitCorpusResult;
        }

        private void add(FicheMeta ficheMeta, FicheFilter ficheFilter, Croisement croisement) {
            groupEngine.add(ficheMeta, ficheFilter, croisement);
        }

    }


    private static class InternalExtraitCorpusResult implements ExtraitCorpusResult {

        private final ExtraitCorpusDef extraitCorpusDef;
        private final SubsetKey corpusKey;
        private final GroupClause groupClause;
        private final FicheGroup[] ficheGroupArray;

        private InternalExtraitCorpusResult(ExtraitCorpusDef extraitCorpusDef, SubsetKey corpusKey, FicheGroup[] ficheGroupArray) {
            this.extraitCorpusDef = extraitCorpusDef;
            this.corpusKey = corpusKey;
            this.groupClause = extraitCorpusDef.getGroupClause();
            this.ficheGroupArray = ficheGroupArray;
        }

        @Override
        public String getName() {
            return extraitCorpusDef.getExtraitName();
        }

        @Override
        public SubsetKey getCorpusKey() {
            return corpusKey;
        }

        @Override
        public boolean withCorpusIntitule() {
            if (corpusKey == null) {
                return false;
            }
            IntituleClause intituleClause = extraitCorpusDef.getIntituleClause();
            if (intituleClause == null) {
                return false;
            }
            return intituleClause.withIntitule();
        }

        @Override
        public String getTagName() {
            return extraitCorpusDef.getTagName();
        }

        @Override
        public PackClause getPackClause() {
            return extraitCorpusDef.getPackClause();
        }

        @Override
        public FicheGroup[] getGroupArray() {
            return ficheGroupArray;
        }

    }

}
