/* BdfServer - Copyright (c) 2017-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.selection;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;
import java.util.TreeSet;
import java.util.function.Predicate;
import net.fichotheque.Subset;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.selection.FicheSelector;
import net.fichotheque.selection.FieldQuery;
import net.fichotheque.selection.MotcleQuery;
import net.fichotheque.selection.MotcleSelector;
import net.fichotheque.selection.SelectionContext;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.utils.Comparators;
import net.fichotheque.utils.CroisementUtils;
import net.fichotheque.utils.selection.FicheSelectorBuilder;
import net.fichotheque.utils.selection.MotcleSelectorBuilder;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.Label;


/**
 *
 * @author Vincent Calame
 */
public final class SelectionEngines {

    private SelectionEngines() {
    }

    public static Collection<FicheMeta> run(SelectionContext selectionContext, Collection<Corpus> corpusCollection) {
        Collection<FicheMeta> result = new ArrayList<FicheMeta>();
        Collection<Corpus> col;
        if (corpusCollection != null) {
            col = corpusCollection;
        } else {
            col = selectionContext.getFichotheque().getCorpusList();
        }
        Predicate<Subset> subsetAccessPredicate = selectionContext.getSubsetAccessPredicate();
        Predicate<FicheMeta> fichePredicate = selectionContext.getFichePredicate();
        for (Corpus corpus : col) {
            if (subsetAccessPredicate.test(corpus)) {
                for (FicheMeta ficheMeta : corpus.getFicheMetaList()) {
                    if (fichePredicate.test(ficheMeta)) {
                        result.add(ficheMeta);
                    }
                }
            }
        }
        return result;
    }

    public static Collection<FicheMeta> run(SelectionContext selectionContext, Collection<Corpus> corpusCollection, Condition condition, MotcleQuery motcleQuery) {
        if ((condition == null) && (motcleQuery == null)) {
            return run(selectionContext, corpusCollection);
        }
        FicheQueryBuilder ficheQueryBuilder = new FicheQueryBuilder();
        if (corpusCollection != null) {
            for (Corpus corpus : corpusCollection) {
                ficheQueryBuilder.addCorpus(corpus);
            }
        }
        if (condition != null) {
            ficheQueryBuilder.setFieldCondition(condition, FieldQuery.TITRE_SCOPE, null);
        }
        if (motcleQuery != null) {
            ficheQueryBuilder.addIndexationMotcleQuery(motcleQuery);
        }
        FicheSelector ficheSelector = FicheSelectorBuilder.init(selectionContext).add(ficheQueryBuilder.toFicheQuery()).toFicheSelector();
        return run(selectionContext, corpusCollection, ficheSelector);
    }

    public static Collection<FicheMeta> run(SelectionContext selectionContext, Collection<Corpus> corpusCollection, FicheQuery ficheQuery) {
        FicheSelector ficheSelector = FicheSelectorBuilder.init(selectionContext).add(ficheQuery).toFicheSelector();
        return run(selectionContext, corpusCollection, ficheSelector);
    }

    public static Collection<FicheMeta> run(SelectionContext selectionContext, Collection<Corpus> corpusCollection, FicheSelector ficheSelector) {
        Predicate<Subset> subsetAccessPredicate = selectionContext.getSubsetAccessPredicate();
        Collection<Corpus> col;
        if (corpusCollection != null) {
            col = corpusCollection;
        } else {
            col = selectionContext.getFichotheque().getCorpusList();
        }
        TreeSet<FicheMeta> treeset = new TreeSet<FicheMeta>(Comparators.FICHEID);
        for (Corpus corpus : col) {
            if (subsetAccessPredicate.test(corpus)) {
                for (FicheMeta ficheMeta : corpus.getFicheMetaList()) {
                    if (ficheSelector.test(ficheMeta)) {
                        treeset.add(ficheMeta);
                    }
                }
            }
        }
        return treeset;
    }

    public static Collection<Motcle> run(SelectionContext selectionContext, Collection<Thesaurus> thesaurusCollection, SubsetKey shouldNotCroisementSubsetKey) {
        Predicate<Subset> subsetAccessPredicate = selectionContext.getSubsetAccessPredicate();
        List<Motcle> result = new ArrayList<Motcle>();
        Collection<Thesaurus> col;
        if (thesaurusCollection != null) {
            col = thesaurusCollection;
        } else {
            col = selectionContext.getFichotheque().getThesaurusList();
        }
        for (Thesaurus thesaurus : col) {
            if (subsetAccessPredicate.test(thesaurus)) {
                appendMotcleList(thesaurus.getFirstLevelList(), result, shouldNotCroisementSubsetKey);
            }
        }
        return result;
    }

    public static Collection<Motcle> run(SelectionContext selectionContext, Collection<Thesaurus> thesaurusCollection, SubsetKey shouldNotCroisementSubsetKey, Condition condition) {
        MotcleQueryBuilder builder = MotcleQueryBuilder.init()
                .setContentCondition(condition, MotcleQuery.WITH_IDALPHA_SCOPE);
        if (thesaurusCollection != null) {
            for (Thesaurus thesaurus : thesaurusCollection) {
                builder.addThesaurus(thesaurus);
            }
        }
        return run(selectionContext, thesaurusCollection, shouldNotCroisementSubsetKey, builder.toMotcleQuery());
    }

    public static Collection<Motcle> run(SelectionContext selectionContext, Collection<Thesaurus> thesaurusCollection, SubsetKey shouldNotCroisementSubsetKey, MotcleQuery motcleQuery) {
        return run(selectionContext, thesaurusCollection, shouldNotCroisementSubsetKey, MotcleSelectorBuilder.init(selectionContext).add(motcleQuery).toMotcleSelector());
    }

    public static Collection<Motcle> run(SelectionContext selectionContext, Collection<Thesaurus> thesaurusCollection, SubsetKey shouldNotCroisementSubsetKey, MotcleSelector motcleSelector) {
        Predicate<Subset> subsetAccessPredicate = selectionContext.getSubsetAccessPredicate();
        Collection<Thesaurus> col;
        if (thesaurusCollection != null) {
            col = thesaurusCollection;
        } else {
            col = motcleSelector.getThesaurusList();
        }
        TreeSet<Motcle> treeSet = new TreeSet<Motcle>(new MotcleComparator(selectionContext.getWorkingLang()));
        for (Thesaurus thesaurus : col) {
            if (subsetAccessPredicate.test(thesaurus)) {
                for (Motcle motcle : thesaurus.getMotcleList()) {
                    if ((!shouldNotTest(motcle, shouldNotCroisementSubsetKey)) && (motcleSelector.test(motcle))) {
                        treeSet.add(motcle);
                    }
                }
            }
        }
        return treeSet;
    }

    private static boolean shouldNotTest(Motcle motcle, SubsetKey shouldNotCroisementSubsetKey) {
        if (shouldNotCroisementSubsetKey == null) {
            return false;
        }
        return CroisementUtils.isShouldNotCroisement(motcle, shouldNotCroisementSubsetKey);
    }

    private static void appendMotcleList(List<Motcle> list, List<Motcle> result, SubsetKey shouldNotCroisementSubsetKey) {
        for (Motcle motcle : list) {
            if (!shouldNotTest(motcle, shouldNotCroisementSubsetKey)) {
                result.add(motcle);
            }
            appendMotcleList(motcle.getChildList(), result, shouldNotCroisementSubsetKey);
        }
    }


    private static class MotcleComparator implements Comparator<Motcle> {

        private final Lang lang;

        public MotcleComparator(Lang lang) {
            this.lang = lang;
        }

        @Override
        public int compare(Motcle mc1, Motcle mc2) {
            String lib1 = toComparableString(mc1, lang);
            String lib2 = toComparableString(mc2, lang);
            int result;
            result = lib1.compareTo(lib2);
            if (result != 0) {
                return result;
            }
            result = mc1.getThesaurus().getSubsetKey().compareTo(mc2.getThesaurus().getSubsetKey());
            if (result != 0) {
                return result;
            }
            int id1 = mc1.getId();
            int id2 = mc2.getId();
            if (id1 < id2) {
                return -1;
            }
            if (id1 > id2) {
                return 1;
            }
            return 0;
        }

    }

    private static String toComparableString(Motcle motcle, Lang lang) {
        String idalpha = motcle.getIdalpha();
        Label label = motcle.getLabels().getLabel(lang);
        if (idalpha != null) {
            if (label != null) {
                return idalpha + " – " + label.getLabelString();
            } else {
                return idalpha;
            }
        } else {
            if (label != null) {
                return label.getLabelString();
            } else {
                return "";
            }
        }
    }

}
