/* FichothequeLib_Xml - Copyright (c) 2011-2019 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.xml.extraction;

import java.io.IOException;
import java.util.Map;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.fiche.FicheItem;
import net.fichotheque.extraction.ExtractionContext;
import net.fichotheque.extraction.def.PackClause;
import net.fichotheque.extraction.run.ExtraitCorpusResult;
import net.fichotheque.extraction.run.FicheExtractInfo;
import net.fichotheque.extraction.run.FicheGroup;
import net.fichotheque.utils.FichothequeUtils;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangContext;
import net.mapeadores.util.localisation.ListLangContext;
import net.mapeadores.util.localisation.UserLangContext;
import net.mapeadores.util.text.LabelUtils;
import net.mapeadores.util.xml.XMLPart;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class ExtraitCorpusXMLPart extends XMLPart {

    private final FicheXMLPart ficheXMLPart;
    private final FicheItemXMLPart ficheItemXMLPart;
    private final PackWriter packWriter = new PackWriter();
    private final ExtractionContext extractionContext;

    public ExtraitCorpusXMLPart(XMLWriter xmlWriter, ExtractionContext extractionContext) {
        super(xmlWriter);
        this.ficheXMLPart = new FicheXMLPart(xmlWriter, extractionContext);
        this.ficheItemXMLPart = new FicheItemXMLPart(xmlWriter);
        ficheItemXMLPart.setCompleteMode(extractionContext);
        this.extractionContext = extractionContext;
    }

    public void addExtraitcorpus(ExtraitCorpusResult extraitCorpusResult) throws IOException {
        String tagName = extraitCorpusResult.getTagName();
        if (tagName != null) {
            startOpenTag(tagName);
            String extraitName = extraitCorpusResult.getName();
            if (extraitName != null) {
                addAttribute("extrait-name", extraitName);
            } else {
                addAttribute("corpus", extraitCorpusResult.getCorpusKey().getSubsetName());
            }
            endOpenTag();
        }
        packWriter.setCurrentPackClause(extraitCorpusResult.getPackClause());
        for (FicheGroup ficheGroup : extraitCorpusResult.getGroupArray()) {
            addGroup(ficheGroup);
        }
        packWriter.flush();
        if (extraitCorpusResult.withCorpusIntitule()) {
            addCorpusIntitule(extraitCorpusResult.getCorpusKey());
        }
        if (tagName != null) {
            closeTag(tagName);
        }
    }

    private void addGroup(FicheGroup group) throws IOException {
        String tagName = group.getTagName();
        if (tagName != null) {
            String name = group.getName();
            startOpenTag(tagName);
            addAttribute("name", name);
            Map<String, String> attributesMap = group.getAttributesMap();
            if (attributesMap != null) {
                for (Map.Entry<String, String> mapEntry : attributesMap.entrySet()) {
                    addAttribute(mapEntry.getKey(), mapEntry.getValue());
                }
            }
            endOpenTag();
            FicheItem[] matchingFicheItemArray = group.getMatchingFicheItemArray();
            if (matchingFicheItemArray != null) {
                for (FicheItem ficheItem : matchingFicheItemArray) {
                    ficheItemXMLPart.addFicheItem(ficheItem);
                }
            }
        }
        if (group.isBottomGroup()) {
            FicheExtractInfo[] array = group.getFicheExtractInfoArray();
            if (array != null) {
                for (FicheExtractInfo ficheExtractInfo : array) {
                    packWriter.ficheStart();
                    ficheXMLPart.addFiche(ficheExtractInfo);
                    packWriter.ficheEnd();
                }
            }
        } else {
            FicheGroup[] subgroupArray = group.getSubgroupArray();
            if (subgroupArray != null) {
                for (FicheGroup ficheGroup : subgroupArray) {
                    addGroup(ficheGroup);
                }
            }
        }
        if (tagName != null) {
            packWriter.groupFlush();
            closeTag(tagName);
        }
    }

    private void addCorpusIntitule(SubsetKey corpusKey) throws IOException {
        Corpus corpus = (Corpus) extractionContext.getFichotheque().getSubset(corpusKey);
        openTag("intitule");
        LangContext langContext = extractionContext.getLangContext();
        if (langContext instanceof ListLangContext) {
            for (ListLangContext.Unit unit : (ListLangContext) langContext) {
                Lang lang = unit.getLang();
                String title = FichothequeUtils.getTitle(corpus, lang);
                XMLUtils.addLibElement(this, lang, title);
                LabelUtils.addLabel(this, lang, title);
            }
        } else if (langContext instanceof UserLangContext) {
            UserLangContext userLangContext = (UserLangContext) langContext;
            Lang workingLang = userLangContext.getWorkingLang();
            String title = FichothequeUtils.getTitle(corpus, workingLang);
            XMLUtils.addLibElement(this, null, title);
            LabelUtils.addLabel(this, null, title);
        }
        closeTag("intitule");
    }


    private class PackWriter {

        private PackClause packClause;
        private String tagName;
        private int packSize;
        private int counter = 0;
        private int modulo = 0;
        private int packCounter = 1;

        PackWriter() {
        }

        void setCurrentPackClause(PackClause newPackClause) {
            reset();
            if (newPackClause != null) {
                String tn = newPackClause.getTagName();
                if (tn != null) {
                    int ps = newPackClause.getPackSize();
                    if (ps > 0) {
                        this.packClause = newPackClause;
                        this.tagName = tn;
                        this.packSize = ps;
                        int m = newPackClause.getModulo();
                        if (m < 2) {
                            m = 0;
                        }
                        this.modulo = m;
                    }
                }
            }
        }

        public void ficheStart() throws IOException {
            if (!testPack()) {
                return;
            }
            if (counter == 0) {
                startOpenTag(tagName);
                if (modulo > 0) {
                    addAttribute("modulo-val", packCounter);
                    if (packCounter == modulo) {
                        packCounter = 1;
                    } else {
                        packCounter++;
                    }
                }
                endOpenTag();
            }
            counter++;
        }

        public void ficheEnd() throws IOException {
            if (!testPack()) {
                return;
            }
            if (counter == packSize) {
                closeTag(tagName);
                counter = 0;
            }
        }

        public void groupFlush() throws IOException {
            if (!testPack()) {
                return;
            }
            if (counter > 0) {
                closePack();
            }
        }

        public void flush() throws IOException {
            if (!testPack()) {
                return;
            }
            if (counter > 0) {
                closePack();
            }
            reset();
        }

        private boolean testPack() {
            return (packClause != null);
        }

        private void closePack() throws IOException {
            closeTag(tagName);
            counter = 0;
        }

        private void reset() {
            packClause = null;
            tagName = null;
            counter = 0;
            packSize = 0;
            modulo = 0;
            packCounter = 1;
        }

    }

}
